<?php
/**
 * CedCommerce
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the End User License Agreement (EULA)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://cedcommerce.com/license-agreement.txt
 *
 * @category    Ced
 * @package     Ced_home
 * @author      CedCommerce Core Team <connect@cedcommerce.com>
 * @copyright   Copyright CEDCOMMERCE (http://cedcommerce.com/)
 * @license     http://cedcommerce.com/license-agreement.txt
 */

namespace App\Bigcommercehome\Components\Order;

use App\Bigcommercehome\Components\Core\Common;
use Exception;

class Helper extends Common
{
    public function createOrderviaMarketplaceOrder($source_order_id, $userId)
    {
        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $order_container = $mongo->getCollectionForTable("order_container");
        $orderData = $order_container->findOne(array('source_order_id' => $source_order_id, 'user_id'=> $userId));
        
        $this->di->getLog()->logContent('order ' . json_encode($orderData), 'info','order.log');
        
        if(count($orderData)==0) return [ 'data'=> "order id not found"];

        if(isset($orderData['target_order_id'])) return ['data'=> 'Order already exist on bigcommerce'];
       
        // $sku = 'ABS'; //$orderData['fulfillments'][0]['line_items'][0]['sku']
        $sku = $orderData['fulfillments'][0]['line_items'][0]['sku'];
        $product_container = $mongo->getCollectionForTable("product_container");
        $productData = $product_container->findOne(array('sku' => $sku, 'user_id'=> $userId));
        
        $this->di->getLog()->logContent('product ' . json_encode($productData), 'info','order.log');
        if(empty($productData) || (is_array($productData) && count($productData)==0)) {
            echo $source_order_id." product was not found on Store.";
            echo PHP_EOL;
            $filter = [ 'source_order_id' => $source_order_id, 'user_id'=> $userId];
            $options=[ '$set' => ["target_error_message"=> 'product was not found on Store.']];
            $response = $order_container->updateOne($filter,$options);
            return ['success'=> false , 'data'=> "product not found"];
        }
        $shop = $this->di->getObjectManager()->get("\App\Core\Models\User\Details")->getDataByUserID($userId, 'bigcommerce');

        if(isset($shop['channel_id']) && $shop['channel_id']!=1){
            $body['channel_id'] = (int)$shop['channel_id'];
        }else if(isset($shop['warehouses'][0]['channel_id']) && $shop['warehouses'][0]['channel_id']!=1){
            $body['channel_id']= (int)$shop['warehouses'][0]['channel_id'];
        }
        
        if($orderData['source_status'] == 'paid'){
            $body['status_id'] = 10;
        }elseif($orderData['source_status'] == 'cancelled'){
            $body['status_id'] = 5;
        }else{
            $body['status_id'] = 10;
        }

        $name = explode(" ",$orderData['billing_address']['full_name']);
       
        if(count($name)==2) {
            $body['billing_address']['first_name']=$name[0];
            $body['billing_address']['last_name']=$name[1];
        }else{
            $body['billing_address']['first_name']=$orderData['billing_address']['full_name'];
            $body['billing_address']['last_name']="";
        }

        $orderEmail = "mercadoorder@bigcommerce.com";
        if(!empty($shop['order_email'])){
            $orderEmail =  $shop['order_email'];
        }else if(!empty($shop['admin_email'])){
            $orderEmail = $shop['admin_email'];
        }
        
        $body['billing_address']['street_1']= $orderData['billing_address']['address1'];
        $body['billing_address']['city']= $orderData['billing_address']['city'];
        $body['billing_address']['state']=$orderData['billing_address']['province'];
        $body['billing_address']['zip']=$orderData['billing_address']['zip'];
        $body['billing_address']['country']=$orderData['billing_address']['country'];
        $body['billing_address']['country_iso2']=$orderData['billing_address']['country_code'];
        $body['billing_address']['email']= $orderEmail;

        $body['shipping_addresses'][0]['street_1']= !empty($orderData['shipping_address']['address1'])?$orderData['shipping_address']['address1']:$orderData['billing_address']['address1'];
        $body['shipping_addresses'][0]['city']= !empty($orderData['shipping_address']['city'])?$orderData['shipping_address']['city']:$orderData['billing_address']['city'];
        $body['shipping_addresses'][0]['state']= !empty($orderData['shipping_address']['province'])?$orderData['shipping_address']['province']:$orderData['billing_address']['province'];
        $body['shipping_addresses'][0]['zip']= !empty($orderData['shipping_address']['zip'])?$orderData['shipping_address']['zip']:$orderData['billing_address']['zip'];
        $body['shipping_addresses'][0]['country']= !empty($orderData['shipping_address']['country'])?$orderData['shipping_address']['country']:$orderData['billing_address']['country'];
        $body['shipping_addresses'][0]['country_iso2']= !empty($orderData['shipping_address']['country_code'])?$orderData['shipping_address']['country_code']:$orderData['billing_address']['country_code'];
        $body['shipping_addresses'][0]['email'] = $orderEmail;

        $containerId =  explode("_", $productData['container_id']);
        $orderedProductId = $containerId[1];
        
        $orderedProductVariantId = $productData['source_product_id'];
        $options = $this->di->getObjectManager()->get("\App\Connector\Components\ApiClient")
            ->init('bigcommerce', true,'bigcommerce_meli')
            ->call('/product/variant/id', [], ['shop_id' => $shop['remote_shop_id'], 'product_id' => $orderedProductId, 'variant_id' => $orderedProductVariantId], 'GET');
        
        $body['products'][0]['product_id']= $orderedProductId;
        if (isset($options['data']['option_values']) && !empty($options['data']['option_values'])) {
            foreach ($options['data']['option_values'] as $optionValues) {
                $body['products'][0]['product_options'][]= array(
                    "id"=> $optionValues['option_id'],
                    "value"=> $optionValues['id'],
                );
            }
        }
        
        // $body['products'][0]['name'] = $orderData['line_items'][0]['title'];

        $body['products'][0]['quantity'] = $orderData['line_items'][0]['quantity_ordered'];
        $body['products'][0]['price_inc_tax'] = $orderData['line_items'][0]['total_price']; //$orderData['total_price'];
        $body['products'][0]['price_ex_tax'] = $orderData['line_items'][0]['total_price']; //$orderData['subtotal_price'];

        $body['shipping_cost_ex_tax'] = $orderData['shipping_cost_details']['cost'];
        $body['shipping_cost_inc_tax'] = $orderData['shipping_cost_details']['cost'];

        $body['total_inc_tax'] = $orderData['total_price'];
        $body['total_ex_tax'] = $orderData['total_price'];

        // $body['subtotal_inc_tax'] = $orderData['total_price'];
        // $body['subtotal_ex_tax'] = $orderData['total_price'];

        $body['customer_message'] =  'Mercado Libre,'.$source_order_id;
        
        $this->di->getLog()->logContent('orderRequest ' . json_encode($body), 'info','order.log');

        $responseOrder = $this->di->getObjectManager()->get("\App\Connector\Components\ApiClient")
            ->init('bigcommerce', true,'bigcommerce_meli')
            ->call('/order/create', [], ['shop_id' => $shop['remote_shop_id'], $body], 'POST');

        $this->di->getLog()->logContent('orderResponse ' . json_encode($responseOrder), 'info','order.log');
        if(isset($responseOrder['data'][0]['message']) && $responseOrder['data'][0]['message']=="The field 'channel_id' is invalid."){
            $body['channel_id'] = 1;
            $responseOrder = $this->di->getObjectManager()->get("\App\Connector\Components\ApiClient")
            ->init('bigcommerce', true,'bigcommerce_meli')
            ->call('/order/create', [], ['shop_id' => $shop['remote_shop_id'],$body], 'POST');

        $this->di->getLog()->logContent('order-R-Response ' . json_encode($responseOrder), 'info','order.log');
        }
    
        if(isset($responseOrder['data'][0]['message'])){
            $error = $responseOrder['data'][0]['message'];
            $filter = [ 'source_order_id' => $source_order_id , 'user_id'=> $userId];
            $options = [ '$set' => ["target_error_message"=> $error]];
            
        }else{
            $target_order_id = $responseOrder['data']['id'];
            echo "target_order_id // "; $target_order_id;
            $filter = [ 'source_order_id' => $source_order_id, 'user_id'=> $userId ];
            $options = [ '$set' => [ 'target_order_id' => $target_order_id, "target_error_message"=> false ]]; 
        }
        $responseData = $order_container->updateOne($filter, $options);
        if($responseData->getModifiedCount()>0)
        return ['success'=>true,'data'=>'Order container updated '];
        else
        return ['success'=>false,'data'=>'Some Error Occured'];
    }

    public function updateOrderviaMarketplaceOrder($source_order_id, $userId)
    {
        $this->di->getLog()->logContent('Level1 ' . $source_order_id, 'info','orderStatusUpdate.log');
        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $order_container = $mongo->getCollectionForTable("order_container");
        $orderData = $order_container->findOne(array('source_order_id' => $source_order_id, 'user_id'=> $userId));
        if(count($orderData)==0) return [ 'data'=> "order id not found"];

        $this->di->getLog()->logContent('Level2 ' . json_encode($orderData), 'info','orderStatusUpdate.log');
        
        $body = [];
        if( $orderData['source_status'] == "cancelled" || (isset($orderData['fulfillments']['shipment_status']) && $orderData['fulfillments']['shipment_status']=="cancelled")) $body['status_id']=5;
        $shop = $this->di->getObjectManager()->get("\App\Core\Models\User\Details")->getDataByUserID($userId, 'bigcommerce');
        
        $this->di->getLog()->logContent('Level3 ' . json_encode($body), 'info','orderStatusUpdate.log');

        if (isset($orderData['target_order_id']) && !empty($orderData['target_order_id']) && !empty($body)) {
            $responseOrder = $this->di->getObjectManager()->get("\App\Connector\Components\ApiClient")
            ->init('bigcommerce', true, 'bigcommerce_meli')
            ->call('/order/update', [], ['shop_id' => $shop['remote_shop_id'],'order_id'=> $orderData['target_order_id'], 'data' => $body], 'PUT');

            $this->di->getLog()->logContent('Level4 ' . json_encode($responseOrder), 'info', 'orderStatusUpdate.log');
            
            if (isset($responseOrder['data'][0]['message'])) {
                return ['success'=>false,'data'=>'Some Error Occured'];
            } else {
                return ['success'=>true,'data'=>'status updated successfully '];
            }
        }else{
            return ['success'=> false,'data'=>'BigCommerce Order Id is not avalaible.'];
        }

    }
}
