<?php
/**
 * CedCommerce
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the End User License Agreement (EULA)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://cedcommerce.com/license-agreement.txt
 *
 * @category    Ced
 * @package     Ced_Bigcommercehome
 * @author      CedCommerce Core Team <connect@cedcommerce.com>
 * @copyright   Copyright CEDCOMMERCE (http://cedcommerce.com/)
 * @license     http://cedcommerce.com/license-agreement.txt
 */

namespace App\Bigcommercehome\Components\Product;

use App\Bigcommercehome\Components\Core\Common;
use Exception;

class Helper extends Common
{

    const BIGCOMMERCE_QUEUE_PRODUCT_IMPORT_MSG = 'BIGCOMMERCE_PRODUCT_IMPORT';

    const BIGCOMMERCE_QUEUE_INV_IMPORT_MSG = 'BIGCOMMERCE_INVENTORY_SYNC';

    public function formatCoreDataForDetail($pData)
    {  
        $temp['user_id'] = $this->di->getUser()->id;
        // $temp['parent_admin_id'] = $this->parentAdminId($temp['user_id']);
        $temp['status'] = "pending";
        
        // if($pData['type'] == "simple"){
        //     $temp['source_product_id'] = (string)'ced_'.$pData['product_id'];
        // }else{
        //     $temp['source_product_id'] = (string)$pData['product_id'];
        // }
        $temp['source_product_id'] = (string)'ced_'.$pData['product_id'];
        // $temp['source_product_id'] = (string)'ced_'.$pData['id'];

        $temp['container_id']  = (string)'ced_'.$pData['product_id'];
        $temp['title'] = isset($pData['name']) ? $pData['name'] : $pData['title'];

        if(!empty($pData['description'])){
            $temp['description'] = $pData['description'];
        }else{
            $temp['description'] = isset($pData['name']) ? $pData['name'] : $pData['title'];
        }
        $temp['type'] = $pData['type'];
        $temp['brand'] = $pData['brand'];
        $temp['vendor'] = $pData['brand'];
        $temp['product_type'] = $pData['product_type'];
        $temp['collection'] = $pData['collection'];
        $temp['additional_images'] = [];
        
        if (isset($pData['images']) && !empty($pData['images'])) {
            foreach ($pData['images'] as $image) {
                array_push($temp['additional_images'], $image);
            }
        }
        
        $temp['is_imported'] = 1;
        $temp['visibility']= "Catalog and Search";
        if(isset($pData['variant_attributes'])) $temp['variant_attributes'] = $pData['variant_attributes'];
        return $temp;
    }

    public function parentAdminId($userId){
        $target = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Shop')->getUserMarkeplace();
        $userData = $this->di->getObjectManager()->get("\App\Core\Models\User\Details")->getDataByUserID($userId, $target);
        $userData=\GuzzleHttp\json_decode(\GuzzleHttp\json_encode($userData),true);
        if(isset($userData['admin_home_id'])){
            return $userData['admin_home_id'];
        }else{
            return $userId;
        }
    }

    public function shapeSqsDataForUpload($data){
        $handlerData = [
            'type' => 'full_class',
            'class_name' => '\App\Bigcommercehome\Components\Product\Route\Requestcontrol',
            'queue_name' => 'product_upload',
            'own_weight' => 100,
            'user_id' => $data['data']['user_id'],
            'data' => $data['data'],
            'run_after' => 5,
            'method' => 'uploadProduct'
        ];
        return $handlerData;
    }

    public function shapeSqsDataForUploadProducts($userId,$remoteShopId,$productCount,$source,$profileId,$sourceShopId,$targetShopId,$productUploadedCount,$sync_status,$filePath,$fileName){
        try{
            $queuedTask = new \App\Connector\Models\QueuedTasks;
            $queuedTask->set([
                'user_id' => $userId,
                'message' => Helper::BIGCOMMERCE_QUEUE_PRODUCT_IMPORT_MSG.' : initializing ...',
                'progress' => 0.00,
                'shop_id' => $userId
            ]);
            $status = $queuedTask->save();
        } catch (Exception $e){
            echo $e->getMessage();
        }
        $handlerData = [
            'type' => 'full_class',
            'class_name' => '\App\Bigcommercehome\Components\Product\Route\Requestcontrol',
            'method' => 'uploadProduct',
            'queue_name' => 'product_upload',
            'own_weight' => 100,
            'user_id' => $userId,
            'data' => [
                'cursor' => 0,
                'uploaded_product_count' => $productUploadedCount,
                'sync_status' => $sync_status,
                'file_path' => $filePath,
                'file_name' => $fileName,
                'success_message' => 'All selected products successfully uploaded to BigCommerce',
                'failure_message' => 'Failed to upload all products to BigCommerce',
                'profileId' => $profileId,
                'source_shop_id' => $sourceShopId,
                'target_shop_id' => $targetShopId,
                'total_count' => $productCount,
                'source' => $source,
                'max_limit' => $productCount,
                'page' => 1,
                'user_id' => $userId,
                'individual_weight' => 0,
                'feed_id' => ($status ? $queuedTask->id : false),
                'pageSize' => 250,
                'remote_shop_id' => $remoteShopId // todo : replace with output function made by anshuman
            ]
        ];
        return $handlerData;
    }

    public function formatCoreDataForVariantData($vData)
    {
        $vData['user_id'] = $this->di->getUser()->id;
        // $vData['parent_admin_id'] = $this->parentAdminId($vData['user_id']);
        $vData['status']="pending";

        if(isset($vData['id']) && !empty($vData['id'])){
            $sourceProductId = $vData['id'];
        }elseif(isset($vData['source_variant_id']) && !empty($vData['source_variant_id'])){
            $sourceProductId = $vData['source_variant_id'];
        }
        // else{
        //     $sourceProductId = $vData['product_id'];
        // }

        // if($vData['type'] == "simple"){
        //     $vData['source_product_id'] = (string)'ced_'.$sourceProductId;
        // }else{
        //     $vData['source_product_id'] = (string)$sourceProductId;
        // }

        // $vData['source_product_id'] = (string)'ced_'.$sourceProductId;
        $vData['source_product_id'] = (string)$sourceProductId;

        $vData['container_id'] = (string)'ced_'.$vData['container_id'];
        $vData['quantity'] = isset($vData['quantity']) ? $vData['quantity'] : "0";
        $vData['price'] = floatval($vData['price']);
        $vData['is_imported'] = 1;
        $date = new \DateTime("now", new \DateTimeZone('GMT') );
        $vData['updated_at'] = $date->format('Y-m-d\TH:i:sO');
        if($vData['type']=="simple"){
            $vData['visibility']= "Catalog and Search";
            // $vData['source_product_id'] = 'ced_'.$vData['source_product_id'];
        }else{
            $vData['visibility']="Not Visible Individually";
        }
        if(isset($vData['offer_price']))$vData['offer_price'] = floatval($vData['offer_price']);
        if(isset($vData['group_id'])) {$vData['group_id'] = (string)'ced_'.$vData['group_id'];}            
        if(isset($vData['id']))unset($vData['id']);

        $rData = [];
        foreach ($vData as $key => $value){
            if($key=="name") {
                $key="title";
            }
                
            if($key=="images") { 
                $key="additional_images";
            }
            if(strpos($key,'$') && strpos($key,'.')){
                // $this->di->getLog()->logContent("Key = ".$key." value = ".$value.' container_id = '.$vData['container_id'], 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'product' . DS . 'test.log');
                $key= str_replace("$","U+FF04",$key);
                $key= str_replace(".","U+FF0E",$key);
            }
            if(strpos($key,'.')){
                $key= str_replace(".","U+FF0E",$key);
            }
            elseif (strpos($key,'$')){
                $key= str_replace("$","U+FF04",$key);
            }
            if($key!="additional_images"){
                $rData[$key] = $value;
            }
            else{
                $rData[$key]=[];
                foreach($value as $image){
                    array_push($rData[$key],$image);
                } 
            }
        }
        return $rData;
    }

    public function getVendorName($userId){
        $target = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Shop')->getUserMarkeplace();
        $userData = $this->di->getObjectManager()->get("\App\Core\Models\User\Details")->getDataByUserID($userId, $target);
        $userData=\GuzzleHttp\json_decode(\GuzzleHttp\json_encode($userData),true);
        return $userData['name'];
    }

    public function shapeSqsData($userId, $remoteShopId, $queuedTaskId)
    {
        $handlerData = [
            'type' => 'full_class',
            'class_name' => '\App\Bigcommercehome\Components\Product\Route\Requestcontrol',
            'method' => 'fetchBigcommerceProduct',
            'queue_name' => 'bigcommerce_product_import',
            'own_weight' => 100,
            'user_id' => $userId,
            'data' => [
                'user_id' => $userId,
                'individual_weight' => 0,
                'feed_id' => $queuedTaskId,
                'pageSize' => 250,
                'remote_shop_id' => $remoteShopId // todo : replace with output function made by anshuman
            ]
        ];
        
        return $handlerData;
    }

    public function getAndQuery($query,$user_id=false){
        $user_id = $user_id ?? $this->di->getUser()->id;
        $andQuery=[
            '$and'=>[
                ['user_id'=>$user_id],
                $query
            ]
        ];
        return $andQuery;
    }
    
    public function getAndQueryMultiple($bulkQuery,$user_id=false){
        $andQuery = $query = [];
        $user_id = $user_id ?? $this->di->getUser()->id;
        foreach ($bulkQuery as $bKey => $bQuery) {
            $query[] = [$bKey=> $bQuery];
        }
        $andQuery=[
            '$and'=>[
                ['user_id'=>$user_id],
                $query[0], $query[1]
            ]
        ];
        
        return $andQuery;
    }

    public function appendMissingWebhookDataFromContainer($data){
        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $mongoCollection = $mongo->setSource("product_container")->getPhpCollection();

        $out = $mongoCollection->aggregate(['$match' => ['source_variant_id' => ['$eq' => $data['source_product_id']]]])->toArray();

        if($out && (count($out) > 0)){
            $temp = json_decode(json_encode($out[0]->variants), true);
            if(isset($temp['collection'])){
                $data['collection'] = $temp['collection'];
            }          
        }
        // $this->di->getLog()->logContent('PROCESS 0000201 | appendMissingWebhookDataFromContainer ','info','shopify'.DS.$this->di->getUser()->id.DS.'product'.DS.date("Y-m-d").DS.'webhook_product_import.log');
        return [
            'data' => $data ,
            'variantAgr' => $out
        ];
    }

    public function ifEligibleSendMail($userId)
    {
        $target = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Shop')->getUserMarkeplace();
        $mongoCollection = $this->di->getObjectManager()->get('\App\Core\Models\BaseMongo');
        $mongoCollection->setSource('user_details');
        $phpMongoCollection = $mongoCollection->getPhpCollection();

        $mongoResponse = $phpMongoCollection->findOne(['user_id' => (string)$userId], ['projection' =>
            ['import_mail' => 1, 'user_id' => 1],
            'typeMap' => ['root' => 'array', 'document' => 'array']]);

        if(!isset($mongoResponse['import_mail'])){
            $phpMongoCollection->updateOne(['user_id' => (string)$userId], ['$set' =>
                ['import_mail' => 1]]);

            $user_details = $this->di->getObjectManager()->get('\App\Core\Models\User\Details');
            $user_details_shopify = $user_details->getDataByUserID((string)$userId, $target);

            $this->di->getObjectManager()->get("\App\Bigcommercehome\Components\Mail\Seller")->imported($user_details_shopify);
        }
    }

    public function resetImportFlag($userId = '')
    {
        $userId = !empty($userId) ? $userId : $this->di->getUser()->id;
        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $collection = $mongo->setSource("product_container")->getPhpCollection();
        try{
            $response = $collection->updateMany(['user_id'=>$userId],['$set' => ['is_imported' => 0]], ['ordered' => false]);
        } catch (Exception $e){}

        return [
            'success' => true,
            'match_count' => isset($response) ? $response->getMatchedCount() : 0,
            'modified_count' => isset($response) ? $response->getModifiedCount() : 0
        ];
    }

    public function deleteNotExistingProduct($userId = '', $append = '')
    {
        $userId = !empty($userId) ? $userId : $this->di->getUser()->id;
        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $collection = $mongo->setSource("product_container_".$userId)->getPhpCollection();

        $query = ['details.is_imported' => 0]; $options = ['projection' => ['details.source_product_id' => 1 ]  ];
        $cursor = $collection->find($query, $options);

        $sourceProductId = [];
        foreach ($cursor as $document) {
            $sourceProductId[] = $document['details']['source_product_id'];
        }
        $mongoData = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Data');
        if(count($sourceProductId)){
            $ids = array_chunk($sourceProductId, 75);
            foreach ($ids as $pid){
                $fbResponse = $this->di->getObjectManager()->get('\App\Facebookhome\Components\Helper')->deleteProduct(['selected_products' => $pid]);
            }
            foreach ($sourceProductId as $id){
                $response = $mongoData->deleteIndividualVariants($id, [], true);
            }
        }

        $options = [];
        $pipeline = [
            [
                '$unwind' => [
                    'path' => '$variants'
                ]
            ],
            [
                '$match' => [
                    'variants.is_imported' => 0
                ]
            ],
            [
                '$project' => [
                    'variants.source_variant_id' => 1,
                    'variants.container_id' => 1
                ]
            ]
        ];
        $cursor = $collection->aggregate($pipeline, $options);
        $sourceVariantId = [];
        $data = [];
        foreach ($cursor as $document) {
            $sourceVariantId[] = $document['variants']['source_variant_id'];
            $data[$document['variants']['source_variant_id']] = $document['variants']['container_id'];
        }

        if(count($sourceVariantId)){
            foreach ($data as $variantId => $containerId){
                $mongoData->deleteIndividualVariants($containerId, [(string)$variantId], false, $userId);
                print_r($mongoData->deleteIndividualVariants($containerId, [(string)$variantId], false, $userId));
            }
        }
    }

        /**
     * @param $response
     * @return Array
     */
    public function shapeProductResponse($data, $userId)
    {
        $userData = $this->di->getObjectManager()->get('\App\Core\Models\User\Details')->getDataByUserID($userId, 'bigcommerce');
        $dimensionUnits = $userData->dimension_units;
        $weightUnits = $userData->weight_units;
        $currency = $userData->currency;

        $formatResponse = [];
        $product_count = $variant_count =0;
        $variant_found = 0;
        foreach ($data as $product) {
            $product['original_weight_units'] = $weightUnits;
            $product['original_dimension_units'] = $dimensionUnits;
            $product['weight_units'] = $this->formatWeightUnits($weightUnits);
            $product['dimension_units'] = $this->formatDimensionUnits($dimensionUnits);
            $product['currency'] = $currency;

            $details = $this->productFormat($product);

            $formatResponse = array_merge($formatResponse, $details);
            // if(count($product['variants']) > 1){
            //     print_r(json_encode($formatResponse));die();
            // }
            $product_count++;
            $variant_count += count($product['variants']);
        }
        $response['data'] = $formatResponse;
        
        $response['count'] = [
            'product' => $product_count,
            'variant' => $variant_count
        ];

        return $response;
    }

    public function formatWeightUnits($unit){
        switch ($unit) {
            case 'KGS':
                $unit = 'kg';
                break;
            case 'LBS':
                $unit = 'lb';
                break;
            case 'Ounces':
                $unit = 'oz';
                break;
            case 'Grams':
                $unit = 'g';
                break;
            case 'Tonnes':
                $unit = 'kg'; //converting to support weight for MercadoLibre
                break;
        }
        return $unit;
    }

    public function formatDimensionUnits($unit){
        switch ($unit) {
            case 'Centimeters':
                $unit = 'cm';
                break;
            case 'Inches':
                $unit = 'in';
                break;
        }
        return $unit;
    }

    private function productFormat($product)
    {
        $formatted_product['type'] = 'simple';
        $methods = [
            'id' => function($product, $value, &$fproduct){
                $fproduct['container_id'] = $value;
            },
            'name' => function($product, $value, &$fproduct){
                $fproduct['title'] = $value;
            },
            'categories' => function($product, $value, &$fproduct){
                $fproduct['product_type'] = isset($value[0])?$value[0]:"";
                $fproduct['tags'] = implode(',', $value);
                $fproduct['collection'] = [];
                foreach($value as $cateName){
                    $fproduct['collection'][] = [
                        "value" => $cateName
                    ];
                }
            },
            'description' => function($product, $value, &$fproduct){
                $fproduct['description'] = $value;
            },
            'brand' => function($product, $value, &$fproduct){
                $fproduct['brand'] = $value;
            },
            'variants' => function($product, $value, &$fproduct){
                $options = current($value)['option_values'];
                if(count($value) > 1) {
                    $fproduct['type'] = 'variation';
                }else if(count($value)==1 && !empty($options)){
                    $fproduct['type'] = 'variation';
                }else{
                    $fproduct['type'] = 'simple';
                }
                foreach ($options as $option) {
                    $fproduct['variant_attributes'][] = strtolower($option['option_display_name']);
                }
            },
            'images' => function($product, $value, &$fproduct){
                if(!empty($value)) {
                    foreach ($value as $image) {
                        $fproduct['images'][] = $image['url_zoom'];
                        if($image['is_thumbnail']){
                            $fproduct['main_image'] = $image['url_zoom'];
                            $fproduct['image_url'] = $image['url_zoom'];
                        }
                    }
                }
            },
            'date_created' => function($product, $value, &$fproduct){
                $fproduct['published_at'] = $value;
            },
            'date_created' => function($product, $value, &$fproduct){
                $fproduct['created_at'] = $value;
            },
            'date_modified' => function($product, $value, &$fproduct){
                $fproduct['updated_at'] = $value;
            },
            'upc' => function($product, $value, &$fproduct){
                if(!empty($product['upc'])){
                    $fproduct['barcode'] = $product['upc'];
                }else if(!empty($product['gtin'])){
                    $fproduct['barcode'] = $product['gtin'];
                }
            },
            'mpn' => function($product, $value, &$fproduct){
                if(!empty($product['sku']) && empty($product['mpn'])){
                    $fproduct['mpn'] = $product['sku'];
                }
            },
            'depth' => function($product, $value, &$fproduct){
                $fproduct['length'] = $product['depth'];
            }
        ];
        
        foreach($product as $key => $value) {
            if (isset($methods[$key])) {
                $methods[$key]($product, $value, $formatted_product);
            } else {
                $formatted_product[$key] = $value;
            }
        }
        if(isset($formatted_product['main_image'])){
            $product['main_image'] = $formatted_product['main_image'];
        }
        $formatted_product['parent_sku'] = $formatted_product['sku'];
        $product['parent_sku'] = $product['sku'];
        $product['parent_id'] = $product['id'];

        $result = [];
        unset($formatted_product['id']);
        unset($formatted_product['sku']);
        unset($product['sku']);
        unset($product['id']);

        $tCount = 0;
        foreach($product['variants'] as $variant) {
            //  if($tCount<2){ //toPerformTest
            $result[] = array_merge($formatted_product, $this->variantFormat($product, $variant, $formatted_product));
            $tCount = $tCount+1;
            //  }
        }
        return $result;
    }

    private function variantFormat($product, $variant, $formatted_product)
    {
        $formatted_variant = [];

        $methods = [
            'id' => function($product, $value, &$fvariant){
                if(count($product['variants']) > 1) {
                    $fvariant['group_id'] = $product['parent_id'];
                }else if(count($product['variants']) == 1){
                    if(isset($product['variants'][0]['option_values']) && !empty($product['variants'][0]['option_values'])){
                        $fvariant['group_id'] = $product['parent_id'];
                    }
                }
                $fvariant['source_variant_id'] = $fvariant['id'] = $value;
            },
            'price' => function($product, $value, $variant, &$fvariant){
                if($variant['calculated_price'] > $variant['price']) {
                    $fvariant['price'] = $variant['calculated_price'];
                } else {
                    if(!empty($variant['calculated_price'])){
                        $fvariant['price'] = $variant['calculated_price'];
                    }else if(!empty($variant['price'])){
                        $fvariant['price'] = $variant['price'];
                    } else{
                        $fvariant['price'] = $product['price'];
                    }
                }
                $fvariant['currency'] = $product['currency'];
            },
            'sale_price' => function($product, $value, $variant, &$fvariant){
                if(!empty($variant['sale_price'])) {
                    $fvariant['offer_price'] = $variant['sale_price'];
                }else if(!empty($product['sale_price'])){
                    $fvariant['offer_price'] = $product['sale_price'];
                }
            },
            'retail_price' => function($product, $value, $variant, &$fvariant){
                if(!empty($variant['retail_price'])) {
                    $fvariant['compareAtPrice'] = $variant['retail_price'];
                }else if(!empty($product['retail_price'])){
                    $fvariant['compareAtPrice'] = $product['retail_price'];
                }
            },
            'inventory_level' => function($product, $value, &$fvariant){

                if($product['inventory_tracking']=='product'){
                    $fvariant['quantity'] = $product['inventory_level'];
                }else if($product['inventory_tracking']=='none'){
                    $fvariant['quantity'] = 999;
                }else{
                    $fvariant['quantity'] = $value;
                }
            },
            'image_url' => function($product, $value, $variant, &$fvariant){
                if(!empty($value)){
                    $fvariant['variant_image'] = $value;
                }else if(!empty($product['main_image'])){
                    $fvariant['variant_image'] = $product['main_image'];
                }
            },
            'option_values' => function($product, $value, &$fvariant){
                foreach ($value as $option) {
                    $fvariant[strtolower($option['option_display_name'])] = $option['label'];
                }
            },
            'date_created' => function($product, $value, &$fvariant){
                $fvariant['created_at'] = $value;
            },
            'date_modified' => function($product, $value, &$fvariant){
                $fvariant['updated_at'] = $value;
            },
            'inventory_tracking' => function($product, $value, &$fvariant){
                $fvariant['inventory_policy'] = $value;
            },
            'upc' => function($product, $value, $variant, &$fvariant){
                if(!empty($variant['upc'])){
                    $fvariant['barcode'] = $variant['upc'];
                }else if(!empty($variant['gtin'])){
                    $fvariant['barcode'] = $variant['gtin'];
                }else if(!empty($product['upc'])){
                    $fvariant['barcode'] = $product['upc'];
                }else if(!empty($product['gtin'])){
                    $fvariant['barcode'] = $product['gtin'];
                }else{
                    $fvariant['barcode'] = $variant['upc'];
                }
            },
            'sku' => function($product, $value, $variant, &$fvariant){
                if(!empty($variant['sku']) && empty($variant['mpn'])){
                    $fvariant['mpn'] = $variant['sku'];
                }
                $fvariant['sku'] = $variant['sku'];
            },
            'width' => function($product, $value, $variant, &$fvariant){
                if(empty($variant['width']) && !empty($product['width'])){
                    $fvariant['width'] = $product['width'];
                }else{
                    $fvariant['width'] = $variant['width'];
                }

                $fvariant['dimension_units'] = $product['dimension_units'];
            },
            'depth' => function($product, $value, $variant, &$fvariant){
                if(!empty($variant['depth'])){
                    $fvariant['length'] = $variant['depth'];
                }else if(!empty($product['depth'])){
                    $fvariant['length'] = $product['depth'];
                }else{
                    $fvariant['length'] = $variant['depth'];
                }
                unset($fvariant['depth']);unset($variant['depth']);
            },
            'height' => function($product, $value, $variant, &$fvariant){
                if(empty($variant['height']) && !empty($product['height'])){
                    $fvariant['height'] = $product['height'];
                }else{
                    $fvariant['height'] = $variant['height'];
                }
            },
            'weight' => function($product, $value, $variant, &$fvariant){
                if(empty($variant['weight']) && !empty($product['weight'])){
                    $fvariant['weight'] = $product['weight'];
                }else{
                    $fvariant['weight'] = $variant['weight'];
                }

                if($product['original_weight_units'] == 'Tonnes'){
                    $convertWeighttoKGS = $this->convertWeightInKiloGram($product['original_weight_units'], $fvariant['weight']);
                    $product['weight'] = $convertWeighttoKGS;
                    $fvariant['weight'] = $convertWeighttoKGS;
                }
                
                $fvariant['weight_units'] = $product['weight_units'];
            }
        ];

        $varianLevelArray = ['price','sale_price','retail_price','upc','sku', 'width','weight','depth','height', 'image_url'];

        foreach($variant as $key => $value) {
            if (isset($methods[$key])) {
                if (in_array($key, $varianLevelArray)) {
                    $methods[$key]($product, $value, $variant, $formatted_variant);
                } else {
                    $methods[$key]($product, $value, $formatted_variant);
                }
            } else {
                $formatted_variant[$key] = $value;
            }
        }        
        return $formatted_variant;
    }

    //not in use
    private function getIdInfo($id)
    {
        if(($pos = strpos($id, '?')) !== false) {
            $id = substr($id, 0, $pos);
        }

        $id = str_replace('gid://bigcommerce/', '', $id);

        $id_info = explode('/', $id);
        return [
            'type'  => $id_info[0],
            'id'    => $id_info[1]
        ];
    }

    private function convertWeightInGram($unit, $weight)
    {
        $grams = null;
        switch ($unit) {
            case 'KILOGRAMS':
                $grams = $weight * 1000;
                break;

            case 'GRAMS':
                $grams = $weight;
                break;

            case 'POUNDS':
                $grams = $weight * 453.6;
                break;

            case 'OUNCES':
                $grams = $weight * 28.3;
                break;
        }

        return $grams;
    }

    private function convertWeightInKiloGram($unit, $weight)
    {
        $kGrams = null;
        switch ($unit) {
            case 'Tonnes':
                $kGrams = $weight * 1000;
                break;
        }

        return $kGrams;
    }
    public function createproductviamarketplace($source_product_id,$user_id){
        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $product_container = $mongo->getCollectionForTable("product_container");
        $productData = $product_container->findOne(array('source_product_id' => $source_product_id, 'user_id'=> $user_id));
        $body=[];
        $body['name']=$productData['title'];
        $body['price']=$productData['price'];
        $body['weight']=$productData['weight'];
        $body['type']="physical";
        $containerId=$productData['container_id'];
        $productVariants=$product_container->findOne(array('container_id'=>$containerId,'user_id'=>$user_id));
        foreach($productVariants as $productVariant){
            if($productVariant['type']!="variation"){
                $body['variants'][]['sku']=$productVariant['sku'];
                $body['variants'][]['option_values'][0]['option_display_name']=$productVariant['variant_attributes'];
                $body['variants'][]['option_values'][0]['label']=$productVariant['color'];
            }

        }
        


    }
}