<?php
/**
 * CedCommerce
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the End User License Agreement (EULA)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://cedcommerce.com/license-agreement.txt
 *
 * @category    Ced
 * @package     Ced_BigCommercehome
 * @author      CedCommerce Core Team <connect@cedcommerce.com>
 * @copyright   Copyright CEDCOMMERCE (http://cedcommerce.com/)
 * @license     http://cedcommerce.com/license-agreement.txt
 */

namespace App\Bigcommercehome\Components\Product;

use App\Bigcommercehome\Components\Core\Common;
use Exception;

class Import extends Common
{
    const MARKETPLACE_CODE = 'bigcommerce';

    public function _construct()
    {
        /* $this->_callName = 'shopify_core';
        parent::_construct(); */
        /*$this->di->getLog()->logContent('Home BigCommerce : indside constructor of Import.php','info','shopify_product_import.log');*/
    }

    public function importControl($productData, $userId, $sqsData){
        $helper = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Helper');
        $totalTime = $totalVariant = 0;
        $returnParam = [];
        try {
            $remote = $helper->shapeProductResponse($productData['data'], $userId);
            $operationStats = $this->pushToProductContainer($remote, 'vistar', $sqsData['data']);
            if (!$operationStats['success']) {
                return [ 'success' => false, 'requeue'=> false , 'complete_progress' => true, 'notice' => $operationStats['message'], 'sqs_data' => $sqsData ];
            }
            $returnParam = ['success'=> true, 'message'=> $remote['count']['variant'].' products including variants imported from BigCommerce Store.'];
        } catch (\Exception $e) {
            $this->di->getLog()->logContent('PROCESS X00400 | Process - Get Product Count from BigCommerce | Bulkimport | buildProgressBar | Error Message  = '.$e->getMessage().' File Path = '. $sqsData['data']['file_path'].' | Probable Error : JSONL file does not exist.', 'info', 'shopify'.DS.$this->di->getUser()->id.DS.'product'.DS.date("Y-m-d").DS.'vistar_product_import.log');
            return [ 'success' => false, 'requeue'=> false , 'complete_progress' => true, 'sqs_data' => $sqsData ];
        }
        return $returnParam;
    }

    public function pushToProductContainer($productData, $mode = '', $additional_data=[])
    {
        $additional_data['app_code'] = 'bigcommerce_meli';
        $helper = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Helper');
        $productContainer = $this->di->getObjectManager()->create('\App\Connector\Models\ProductContainer');
        $productContainer->setSource("product_container");

        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $mongo->setSource("product_container");
        $mongoCollection = $mongo->getPhpCollection();

        $simpleProductCounter = 0;

        $bulkOpArray = $bulkOpArray2 = $preventToUpdateProduct = [];
        $userId = $this->di->getUser()->id;

        $target = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Shop')->getUserMarkeplace();

        foreach ($productData['data'] as $productData){
            
            $simpleToVariantFlag = false;
            $existsSimpleContainer = [];

            $simpleContainerId = (string)'ced_'.$productData['product_id'];
            $bigProductId = isset($productData['id'])?$productData['id']:$productData['source_variant_id'];
            $bigProductId = (string)$bigProductId;

            $bigContainerId = isset($productData['product_id'])?$productData['product_id']:$productData['container_id'];
            $bigContainerId = (string)'ced_'.$bigContainerId;

            $exists = $mongoCollection->findOne(['source_product_id' => $bigProductId, 'user_id' => (string)$userId]);
            if(!empty($exists)){
                
                $product = [];
                $productData['db_action'] = "variant_update";
                $data = $helper->formatCoreDataForVariantData($productData);
                
                $uniqueProductId = $productData['id'].$productData['container_id'];
                if (!in_array($uniqueProductId, $preventToUpdateProduct)) {
                    $product = $helper->formatCoreDataForDetail($productData);
                }
                // code by himanshu
                if(isset($additional_data['app_code'])) {
                    $app_code = $additional_data['app_code'];
                    // $existingProduct = $exists->getArrayCopy();
                    $existingProduct = json_decode(json_encode($exists), true);
                    if(isset($existingProduct['app_codes'])) {
                        if(!in_array($app_code, $existingProduct['app_codes'])) {
                            $existingProduct['app_codes'][] = $app_code;
                            $data['app_codes'] = $existingProduct['app_codes'];
                        }
                    }
                    else {
                        $data['app_codes'] = [$app_code];
                    }
                }
                // end

                $query = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Helper')->getAndQuery(['source_product_id' => $data['source_product_id']],$userId);
                
                $bulkOpArray[] = [
                    'updateOne' => [
                        $query,
                        [ '$set' => $data]
                    ]
                ];

                if (!empty($product)) {
                    $queryProduct = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Helper')->getAndQueryMultiple(['source_product_id' => (string)$product['source_product_id'], 'container_id' => (string)$product['container_id']], $userId);
                    $bulkOpArray2[] = [
                        'updateOne' => [
                            $queryProduct,
                            [ '$set' => $product]
                        ]
                    ];
                    $preventToUpdateProduct[] = $uniqueProductId;
                }
            } else {
                if ($productData['type'] == "variation") {
                    $existsSimpleContainer = $mongoCollection->findOne(['container_id' => $simpleContainerId, 'user_id' => (string)$userId, 'type' => 'simple']); // check for simple product
                    // if(!empty($existsSimpleContainer)){
                    //     $deleteSimpleContainer = $mongoCollection->deleteOne(['container_id' => $simpleContainerId, 'user_id' => (string)$userId, 'type' => 'simple']);    
                    // }
                }

                $data['details'] = $helper->formatCoreDataForDetail($productData);
                if(!empty($existsSimpleContainer)){
                    $data['is_wrapper_update'] = true;
                    $data['details']['_id'] = $existsSimpleContainer['_id'];
                }
                
                $data['variant_attribute'] = isset($productData['variant_attributes']) ? $productData['variant_attributes'] : [];
                $variants = $helper->formatCoreDataForVariantData($productData);
                
                // code by himanshu
                if(isset($additional_data['app_code'])) {
                    $app_code = $additional_data['app_code'];
                    $data['details']['app_codes'] = [$app_code];
                    $variants['app_codes'] = [$app_code];
                }
                // end
                
                $data['variants'][] = $variants;
                
                $res = $productContainer->createProductsAndAttributes([$data], $target, 0, $userId);
                if(isset($res[0]['success']) && $res[0]['success']) $simpleProductCounter++;
            }

            $data = [];
        }
        
        try{
            
            $returenRes = [];
            if(!empty($bulkOpArray)){
                $bulkObj = $mongoCollection->BulkWrite($bulkOpArray, ['w' => 1]);
                $returenRes[] = [
                    'acknowledged' => $bulkObj->isAcknowledged(),
                    'inserted' => $bulkObj->getInsertedCount(),
                    'modified' => $bulkObj->getModifiedCount(),
                    'matched' => $bulkObj->getMatchedCount(),
                    'mainProductCount' => $simpleProductCounter
                ];
            }

            if (!empty($bulkOpArray2)) {
                $bulkObj2 = $mongoCollection->BulkWrite($bulkOpArray2, ['w' => 1]);
                $returenRes[] = [
                    'acknowledged' => $bulkObj2->isAcknowledged(),
                    'inserted' => $bulkObj2->getInsertedCount(),
                    'modified' => $bulkObj2->getModifiedCount(),
                    'matched' => $bulkObj2->getMatchedCount(),
                    'mainProductCount' => $simpleProductCounter
                ];
            }

            return [
                'success' => true,
                'stats' => $returenRes
            ];
        } catch (\Exception $e)
        {
            return ['success' => false, 'message' => $e->getMessage()];
        }


        return [
            'success' => true,
            'stats' => $simpleProductCounter. ' main product imported successfully'
        ];
    }

    public function getBigcommerceProducts($productData, $userId, $sqsData, $mode = '')
    {
        $additional_data = $sqsData['data'];

        $helper = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Helper');

        $productContainer = $this->di->getObjectManager()->create('\App\Connector\Models\ProductContainer');
        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $mongo->setSource("product_container");
        $mongoCollection = $mongo->getPhpCollection();
            
        $tempCount = $mainProduct = $variant = $time = $totalTime = $totalCount = $simpleProductCounter = 0;
        foreach ($productData['data'] as $productData){
            $totalCount++;
            $query = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Helper')->getAndQuery(['source_product_id' => (string)$productData['id']], $userId);
            $exists = $mongoCollection->findOne($query);
            
            if(!empty($exists)){   
                $productData['marketplace'] = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Shop')->getUserMarkeplace();
                $productData['db_action'] = "variant_update";
                $data = $helper->formatCoreDataForVariantData($productData);
                // code by himanshu
                if(isset($additional_data['app_code'])) {
                    $app_code = $additional_data['app_code'];
                    // $existingProduct = $exists->getArrayCopy();
                    $existingProduct = json_decode(json_encode($exists), true);
                    if(isset($existingProduct['app_codes'])) {
                        if(!in_array($app_code, $existingProduct['app_codes'])) {
                            $existingProduct['app_codes'][] = $app_code;
                            $data['app_codes'] = $existingProduct['app_codes'];
                        }
                    }
                    else {
                        $data['app_codes'] = [$app_code];
                    }
                }
                // end
                $query = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Helper')->getAndQuery(['source_product_id' => $data['source_product_id']],$this->di->getUser()->id);
                $bulkOpArray[] = [
                    'updateOne' => [
                        $query,
                        [ '$set' => $data]
                    ]
                ];
            } else {
                $data = $helper->formatCoreDataForDetail($productData);
                
                // code by himanshu
                if(isset($additional_data['app_code'])) {
                    $app_code = $additional_data['app_code'];

                    $data['details']['app_codes'] = [$app_code];
                    $variants['app_codes'] = [$app_code];
                }
                // end

                $data['variants'][] = $variants;

                $target = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Shop')->getUserMarkeplace();
                $productContainer->setSource("product_container");
                $res = $productContainer->createProductsAndAttributes([$data], $target, 0, $this->di->getUser()->id);
                if(isset($res[0]['success']) && $res[0]['success']) $simpleProductCounter++;
            }
            $data = [];
        }

        try{
            if(!empty($bulkOpArray)){
                $bulkObj = $mongoCollection->BulkWrite($bulkOpArray, ['w' => 1]);
                $returenRes = [
                    'acknowledged' => $bulkObj->isAcknowledged(),
                    'inserted' => $bulkObj->getInsertedCount(),
                    'modified' => $bulkObj->getModifiedCount(),
                    'matched' => $bulkObj->getMatchedCount(),
                    'mainProductCount' => $simpleProductCounter
                ];
                $this->di->getLog()->logContent(' PROCESS 00310 | Import | pushToProductContainer | Bulk Product Write Response : '.print_r($returenRes, true),'info','shopify'.DS.$this->di->getUser()->id.DS.'product'.DS.date("Y-m-d").DS.$mode.'_product_import.log');
                return [
                    'success' => true,
                    'stats' => $returenRes
                ];
            }
            
        } catch (\Exception $e)
        {
            return ['success' => false, 'message' => $e->getMessage()];
        }

        return [
            'success' => true,
            'message' => 'product imported'
        ];
    }

    public function validateAndDeleteVariants($incomingWebhookProductIds, $containerId){
        $mongoData = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Data');
        $currentVariantIds = $mongoData->getallVariantIds($containerId);
        $deleteIds = array_diff($currentVariantIds, $incomingWebhookProductIds);
        $this->di->getLog()->logContent('Delete variants(s) Ids'.print_r($deleteIds,true)." incomingWebhookProductIds = ".print_r($incomingWebhookProductIds,true)." currentVariantIds = ".print_r($currentVariantIds,true),'info','shopify'.DS.$this->di->getUser()->id.DS.'product'.DS.date("Y-m-d").DS.'webhook_product_import.log');
        if(!empty($deleteIds)){
            $this->di->getLog()->logContent('PROCESS 000030 | Import | updateProductandKeepLocation | calling facebook to delete variants(s) ','info','shopify'.DS.$this->di->getUser()->id.DS.'product'.DS.date("Y-m-d").DS.'webhook_product_import.log');

            $this->di->getLog()->logContent('Delete variants(s) Ids = '.print_r($deleteIds,true)." incomingWebhookProductIds = ".print_r($incomingWebhookProductIds,true)." currentVariantIds = ".print_r($currentVariantIds,true),'info','shopify'.DS.$this->di->getUser()->id.DS.'product'.DS.date("Y-m-d").DS.'webhook_product_import.log');

            if(count($currentVariantIds)-count($deleteIds)==1){
                $mongoData->deleteIndividualVariants($containerId, $deleteIds,true);
                $mongoData->deleteIndividualVariants($containerId, $deleteIds);
            }else{
                $mongoData->deleteIndividualVariants($containerId, $deleteIds);
            }
            /*$fbResponse = $this->di->getObjectManager()->get('\App\Facebookhome\Components\Helper')->deleteVariant(['selected_variants' => $deleteIds]);*/
            $this->di->getLog()->logContent('PROCESS 000031 | Import | updateProductandKeepLocation | facebook Response to delete variants(s) ','info','shopify'.DS.$this->di->getUser()->id.DS.'product'.DS.date("Y-m-d").DS.'webhook_product_import.log');
        }
    }

    public function productEnableDisable($data,$userId){
        $userId = $userId ?? $this->di->getUser()->id;
        if(!$userId) return [ 'success' => false, 'message' => 'Invalid User. Please check your login credentials' ];
        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $this->_collection = $mongo->setSource("product_container")->getPhpCollection();
        if(isset($data['all'])){
            $query = ['user_id' => (string)$userId];
            $this->_collection->updateMany($query,['$set' => ['flag' =>(string)$data['all']]]);
            $response=['success'=>true,'msg'=>"All products are ".$data['all']."d"];
        }
        if(isset($data['source_product_ids']) && count($data['source_product_ids'])>0 && isset($data['flag'])){
            foreach ($data['source_product_ids'] as $source_product_id){
                $query = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Helper')->getAndQuery(['source_product_id' => (string)$source_product_id],$userId);
                $this->_collection->updateOne(
                    $query,
                    ['$set' => ['flag' =>$data['flag']]]
                );
                $response=['success'=>true,'msg'=>"Product is ".$data['flag']."d"];
            }
        }
        if(isset($response)){
            return $response;
        }
    }

    public function exportProduct($data,$userId){
        $type=$data['export_type'];
        switch($type){
            case 'xls' :
                $result1=shell_exec('docker-compose exec -T mongo mongoexport -u root -p cedcommerce --authenticationDatabase admin -d home -c product_container --fields type,container_id,title,brand,product_type,description,handle,tags,template_suffix,main_image,published_at,created_at,updated_at,additional_images,collection,variant_title,position,sku,offer_price,price,quantity,weight,weight_unit,grams,barcode,inventory_policy,taxable,fulfillment_service,variant_image,inventory_item_id,inventory_tracked,requires_shipping,locations,user_id,source_product_id,is_imported,visibility,source_marketplace,upload_status,db_action,marketplace,flag --type=excel -q "{\"user_id\":\"283\"}" --out=/home/cedcoss/Documents/csvfiles/productcsv.xlsx  --authenticationDatabase admin');
                break;
            default :
                $result1=shell_exec('docker-compose exec -T mongo mongoexport -u root -p cedcommerce --authenticationDatabase admin -d home -c product_container --fields type,container_id,title,brand,product_type,description,handle,tags,template_suffix,main_image,published_at,created_at,updated_at,additional_images,collection,variant_title,position,sku,offer_price,price,quantity,weight,weight_unit,grams,barcode,inventory_policy,taxable,fulfillment_service,variant_image,inventory_item_id,inventory_tracked,requires_shipping,locations,user_id,source_product_id,is_imported,visibility,source_marketplace,upload_status,db_action,marketplace,flag --type=csv -q "{\"user_id\":\"283\"}" --out=csvfiles/productcsv.csv  --authenticationDatabase admin');
                break;
        }
    }

    public function pushToQueue($data, $time = 0, $queueName = 'temp')
    {
        $data['run_after'] = $time;
        if($this->di->getConfig()->get('enable_rabbitmq_internal')){
            $helper = $this->di->getObjectManager()->get('\App\Rmq\Components\Helper');
            return $helper->createQueue($data['queue_name'],$data);
        }
        return true;
    }

}