<?php
/**
 * CedCommerce
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the End User License Agreement (EULA)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://cedcommerce.com/license-agreement.txt
 *
 * @category    Ced
 * @package     Ced_Shopifyhome
 * @author      CedCommerce Core Team <connect@cedcommerce.com>
 * @copyright   Copyright CEDCOMMERCE (http://cedcommerce.com/)
 * @license     http://cedcommerce.com/license-agreement.txt
 */

namespace App\Bigcommercehome\Components\Product\Inventory;

use App\Bigcommercehome\Components\Core\Common;
use Exception;

class Data extends Common
{
    public $_collection = '';

    public $_error = false;

    const CHUNK_SIZE = 40;


    public function prepare($userId){
        $userId = $userId ?? $this->di->getUser()->id;

        if(!$userId){
            return $this->_error = [
                'success' => false,
                'message' => 'Invalid User. Please check your login credentials'
            ];
        }
        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $mongo->setSource("product_container");
        $this->_collection = $mongo->getPhpCollection();
    }

    public function fetchLocationsInChunk($data, $userId = false){
        $this->prepare($userId);
        $target = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Shop')->getUserMarkeplace();
        $cursor = isset($data['data']['cursor']) ? $data['data']['cursor'] : 0;
        if($this->_error) return $this->_error;

        $invResponse = $this->_collection->distinct(
            'inventory_item_id', ['$and'=>[
            ['user_id' => $userId],
            ["source_marketplace" => $target]
        ]]);
        $arrayChunks = array_chunk($invResponse, Data::CHUNK_SIZE);
        print_r($arrayChunks);
        if($cursor == 0) return [
            'success' => true,
            'locations' => $arrayChunks[$cursor],
            'total_chunk' => count($arrayChunks),
            'total' => count($invResponse)
        ];        
        return [
            'success' => true,
            'locations' => isset($arrayChunks[$cursor]) ? $arrayChunks[$cursor] : 'partial_error'
        ];        
    }

    public function deleteLocations($userId = null , $locationId = []){
        $this->prepare($userId);
        $query = $this->di->getObjectManager()->get('\App\Shopifyhome\Components\Product\Helper')->getAndQuery(["locations" => ['$exists' => true]],$userId);
        try{
            $delete = $this->_collection->updateMany(
                $query,
                [ '$unset' => [
                    "locations" => []
                ]
                ]
            );

        } catch(\ Exception $e){
            echo $e->getMessage();

        }        
    }

    /*public function updateInvAtLocation($data, $userId = null){
        $this->prepare($userId);

        $out = $this->_collection->aggregate([
                ['$match' => [
                    'variants.locations.inventory_item_id' => ['$eq' => (string)$data['inventory_item_id']],
                    'variants.locations.location_id' => ['$eq' => (string)$data['location_id']]
                ]],
                ['$unwind' => '$variants'],
                [ '$unwind' => '$variants.locations'],
                ['$match' => [
                    'variants.locations.inventory_item_id' => ['$eq' => (string)$data['inventory_item_id']],
                    'variants.locations.location_id' => ['$eq' => (string)$data['location_id']]
                ]],
                ['$project' => ['variants.locations.location_id' => 1 ]]
            ])->toArray();



        if(empty($out)){
                $update = $this->_collection->updateOne(
                    [
                        'variants.inventory_item_id' => (string)$data['inventory_item_id']
                    ],
                    [
                        '$push' => [
                            'variants.$.locations' => $data
                        ]
                    ]
                );
        } else {
                $update = $this->_collection->updateOne(
                    [
                        'variants.locations.inventory_item_id' => (string)$data['inventory_item_id']
                    ],
                    [
                        '$set' => [
                            'variants.$.locations.$[loc].available' => $data['available']
                        ]
                    ],
                    ['arrayFilters' => [
                        [
                            'loc.location_id' => ['$eq' => (string)$data['location_id']]]
                    ]
                    ]
                );
        }

        //$this->di->getLog()->logContent('inventory location update , data ='. json_encode($data),'info','webhook_updateInvAtLocation.log');

        return [
            'success' => true,
            'matched_count' => $update->getMatchedCount(),
            'modified_count' => $update->getModifiedCount()
        ];
        //$this->di->getLog()->logContent('matched count = '.$update->getMatchedCount(). "modified count = ".$update->getModifiedCount(),'info','webhook_updateInvAtLocation.log');

    }*/


    public function updateInvAtLocation($data, $userId = null){
        $this->prepare($userId);
        $out = $this->_collection->aggregate([
            ['$match' => [
                '$and'=>[
                    ['user_id'=>(string)$userId],
                    ['locations.inventory_item_id' => ['$eq' => (string)$data['inventory_item_id']]],
                    ['locations.location_id' => ['$eq' => (string)$data['location_id']]]
                ]
            ]],
            [ '$unwind' => '$locations'],
            ['$project' => ['locations.location_id' => 1 ]]
        ]);

        if(empty($out)){
            $query = $this->di->getObjectManager()->get('\App\Shopifyhome\Components\Product\Helper')->getAndQuery(['inventory_item_id' => (string)$data['inventory_item_id']],$userId);
            $update = $this->_collection->updateOne(
                $query,
                [
                    '$push' => [
                        'locations' => $data
                    ]
                ]
            );
        } else {
            $query = $this->di->getObjectManager()->get('\App\Shopifyhome\Components\Product\Helper')->getAndQuery( ['locations.inventory_item_id' => (string)$data['inventory_item_id']],$userId);
            $update = $this->_collection->updateOne(
                $query,
                [
                    '$set' => [
                        'locations.$.available' => $data['available']
                    ]
                ]
            );
        }

        return [
            'success' => true,
            'matched_count' => $update->getMatchedCount(),
            'modified_count' => $update->getModifiedCount()
        ];
    }


    public function getTotalQtyFromLocations($inventoryItemId){
        $query = $this->di->getObjectManager()->get('\App\Shopifyhome\Components\Product\Helper')->getAndQuery(['locations.inventory_item_id' => ['$eq' => (string)$inventoryItemId]],$this->di->getUser()->id);
        $out = $this->_collection->aggregate([
            ['$match' => $query],
            [ '$unwind' => '$locations'],
            ['$match' => $query],
            ['$project' => ['locations.available' => 1 ]]
        ])->toArray();

        $total = 0;
        foreach ($out as $inv){
            $total += $inv['locations']['available'];
        }
        print_r("Total");
        print_r($total);
        return [
            'success' => true,
            'total_qty' => $total
        ];
    }

    public function updateTotalQty($inventoryItemId, $qty)
    {
        $this->_collection->updateOne(
            [
                'inventory_item_id' => (string)$inventoryItemId
            ],
            [
                '$set' => [
                    'quantity' => (int)$qty
                ]
            ]
        );
    }

    public function updateProductVariantInv($invId, $data, $userId = null){
        $this->prepare($userId);
        $query = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Helper')->getAndQuery([ 'inventory_item_id' => $invId],$userId);
        $update = $this->_collection->updateOne(
                        $query,
                        [ '$set' => [
                            "locations" => $data
                            ]
                        ]
                        );
        /* $this->di->getLog()->logContent('inventory item id = '.$data['inventory_item_id'].' Matched %d document(s) : '.$update->getMatchedCount() .'Modified %d document(s) : '.$update->getModifiedCount(),'info','Data.log');  */       
    }
}