<?php
/**
 * CedCommerce
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the End User License Agreement (EULA)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://cedcommerce.com/license-agreement.txt
 *
 * @category    Ced
 * @package     Ced_Shopifyhome
 * @author      CedCommerce Core Team <connect@cedcommerce.com>
 * @copyright   Copyright CEDCOMMERCE (http://cedcommerce.com/)
 * @license     http://cedcommerce.com/license-agreement.txt
 */

namespace App\Shopifyhome\Components\Product\Inventory;

use App\Shopifyhome\Components\Core\Common;
use Exception;

class Hook extends Common
{
    public function updateProductInventory($data, $userId = null){
        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $collection = $mongo->getCollectionForTable('product_container');
        $query = $this->di->getObjectManager()->get('\App\Shopifyhome\Components\Product\Helper')->getAndQuery(['inventory_item_id' => (string)$data['data']['inventory_item_id']],$this->di->getUser()->id);
        $res=$collection->find($query)->toArray();
        if(empty($res)) {
            $data['data']['retry'] = isset($data['data']['retry']) ? (int)($data['data']['retry'] + 1) : 1;
            $this->di->getLog()->logContent($data['data']['inventory_item_id'].' Inv Update Retry Count = '.$data['data']['retry'],'info','shopify' . DS . $this->di->getUser()->id . DS . 'inventory' . DS . date("Y-m-d") . DS . 'retry_inventory_update.log');
            return ['success' => true, 'requeue' => true, 'sqs_data' => $data];
        }else {
            $this->di->getLog()->logContent('PROCESS 000000 | Inventory | Hook | init Inv update | webhook data :' . print_r($data['data'], true), 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'inventory' . DS . date("Y-m-d") . DS . 'webhook_inventory_update.log');

            $productInv = $this->di->getObjectManager()->get("App\Shopifyhome\Components\Product\Inventory\Utility")->typeCastElement($data['data']);


            $updateShopifyProductInv = $this->di->getObjectManager()->get("\App\Shopifyhome\Components\Product\Inventory\Data")->updateInvAtLocation($productInv, $userId);

            $this->di->getLog()->logContent('PROCESS 000001 | Inventory | Hook | init inv update | update shopify product inv  response :' . json_encode($updateShopifyProductInv), 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'inventory' . DS . date("Y-m-d") . DS . 'webhook_inventory_update.log');

            if (($updateShopifyProductInv['matched_count'] == 0) || ($updateShopifyProductInv['modified_count'] == 0)) {
                $this->di->getLog()->logContent('PROCESS 000003 | Inventory | Hook | Matched Count : ' . $updateShopifyProductInv['matched_count'] . ' | Modified Count : ' . $updateShopifyProductInv['modified_count'], 'info', 'shopify' . DS . 'global' . DS . date("Y-m-d") . DS . $this->di->getUser()->id . DS . 'ZeroMatch.log');
            }
            /*if(isset($updateShopifyProductInv['success']) && $updateShopifyProductInv['modified_count'] == 1){*/
            if (isset($updateShopifyProductInv['success'])) {

                $totalQty = $this->di->getObjectManager()->get("\App\Shopifyhome\Components\Product\Inventory\Data")->getTotalQtyFromLocations((string)$data['data']['inventory_item_id']);

                if (isset($totalQty['success'])) {
                    $totalQty = $this->di->getObjectManager()->get("\App\Shopifyhome\Components\Product\Inventory\Data")->updateTotalQty((string)$data['data']['inventory_item_id'], $totalQty['total_qty']);
                }

                $mongoCollection = $this->di->getObjectManager()->get('\App\Core\Models\BaseMongo');
                $mongoCollection->setSource('product_container_' . $this->di->getUser()->id);
                $phpMongoCollection = $mongoCollection->getCollection();
                $out = $phpMongoCollection->aggregate([['$match' => ['variants.inventory_item_id' => ['$eq' => (string)$data['data']['inventory_item_id']]]], ['$unwind' => '$variants'], ['$match' => ['variants.inventory_item_id' => ['$eq' => (string)$data['data']['inventory_item_id']]]], ['$project' => ['variants.source_variant_id' => 1]]])->toArray();

                if (isset($out[0]['variants']['source_variant_id'])) {
                    $this->di->getLog()->logContent('PROCESS 000003 | Inventory | Hook | init inv update | calling fb to update product inv', 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'inventory' . DS . date("Y-m-d") . DS . 'webhook_inventory_update.log');

                    $fbResponse = $this->di->getObjectManager()->get('\App\Facebookhome\Components\Helper')->updateVariants(['selected_variants' => [$out[0]['variants']['source_variant_id']]]);

                    $this->di->getLog()->logContent('PROCESS 000004 | Inventory | Hook | init inv update | fb response :' . json_encode($fbResponse), 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'inventory' . DS . date("Y-m-d") . DS . 'webhook_inventory_update.log');

                } else {
                    $this->di->getLog()->logContent('PROCESS 000005 | Inventory | Hook | init inv update | variant id not found', 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'inventory' . DS . date("Y-m-d") . DS . 'webhook_inventory_update.log');
                }

            } else {
                $this->di->getLog()->logContent('PROCESS 000006 | Inventory | Hook | init inv update | No product has been updated', 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'inventory' . DS . date("Y-m-d") . DS . 'webhook_inventory_update.log');
            }
        }
        return true;
    }

    public function deleteandUpdateProductLocationInventory($data){

    }

}