<?php
/**
 * CedCommerce
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the End User License Agreement (EULA)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://cedcommerce.com/license-agreement.txt
 *
 * @category    Ced
 * @package     Ced_Shopifyhome
 * @author      CedCommerce Core Team <connect@cedcommerce.com>
 * @copyright   Copyright CEDCOMMERCE (http://cedcommerce.com/)
 * @license     http://cedcommerce.com/license-agreement.txt
 */

namespace App\Bigcommercehome\Components\Product\Vistar;

class Import extends \App\Core\Components\Base
{
    const MARKETPLACE_CODE = 'bigcommerce';

    const SHOPIFY_QUEUE_PRODUCT_IMPORT_MSG = 'BIGCOMMERCE_PRODUCT_IMPORT';

    const SHOPIFY_QUEUE_INV_IMPORT_MSG = 'BIGCOMMERCE_INVENTORY_SYNC';

    /*const PAGE_SIZE = 3000;*/
    const PAGE_SIZE = 500;

    const IMPORT_FEED_WEIGHT = 90;


    private $_apiClient;
    private $_connectorHelper;
    private $_vistarHelper;
    private $_vistarData;
    private $_vistarUtility;
    private $_target;

    public function _construct()
    {
        parent::_construct();
    }

    public function init()
    {
        $this->_apiClient = $this->di->getObjectManager()->get('\App\Connector\Components\ApiClient');
        $this->_connectorHelper = $this->di->getObjectManager()->get('\App\Connector\Components\Helper');
        $this->_vistarHelper = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Vistar\Helper');
        $this->_vistarData = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Vistar\Data');
        $this->_vistarUtility = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Vistar\Utility');
        print_r("init");

        $this->_target = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Shop')->getUserMarkeplace();
        return $this;
    }

    public function createRequest($remoteShopId, $sqsData)
    {   
        $filter['shop_id'] = $remoteShopId;
        // $filter['type'] = 'product';
        $remoteResponse = $this->_apiClient->init($this->_target, 'true')->call('product', [], $filter, 'GET');
        // $remoteResponse = $this->_apiClient->init($this->_target, 'true')->call('/bulk/operation', [], $filter, 'POST');
        // print_r($remoteResponse['data']['marketplace']);die;

        $this->di->getLog()->logContent('PROCESS 0001 | Create Bulk Request on Shopify | Import | createRequest | Response from Remote Server = \n' . print_r($remoteResponse, true), 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'product' . DS . date("Y-m-d") . DS . 'vistar_product_import.log');
        
        if (!isset($remoteResponse['success']) || !$remoteResponse['success']) {
            return ['success' => false, 'requeue' => false, 'complete_progress' => true, 'sqs_data' => $sqsData];
        }
        
        if ($remoteResponse['success'] && !isset($remoteResponse['error'])) {
            $msg = Import::SHOPIFY_QUEUE_PRODUCT_IMPORT_MSG . ' : Product(s) import Request is accepted by Shopify.';
            $progress = $this->_connectorHelper->updateFeedProgress($sqsData['data']['feed_id'], $sqsData['data']['individual_weight'], $msg);
            
            $sqsData['data']['operation'] = 'get_status';
            $sqsData['data']['id'] = $remoteResponse['data']['id'];
            $sqsData['data']['individual_weight'] = 1;
            print_r($remoteResponse['data']['status']);die;
            
            return ['success' => true, 'requeue' => true, 'sqs_data' => $sqsData];

        } elseif(isset($remoteResponse['error']['message']) && strpos($remoteResponse['error']['message'], 'A bulk operation for this app and shop is already in progress:') !== false){
            $gIdData = explode('A bulk operation for this app and shop is already in progress:', $remoteResponse['error']['message']);
            $gId = trim(substr($gIdData[1], 0, -1));

            $msg = Import::SHOPIFY_QUEUE_PRODUCT_IMPORT_MSG . ' : Product(s) import Request is accepted by Shopify.';
            $progress = $this->_connectorHelper->updateFeedProgress($sqsData['data']['feed_id'], $sqsData['data']['individual_weight'], $msg);

            $sqsData['data']['operation'] = 'get_status';
            $sqsData['data']['id'] = $gId;
            $sqsData['data']['individual_weight'] = 1;

            return ['success' => true, 'requeue' => true, 'sqs_data' => $sqsData];
        }
        else return ['success' => false, 'requeue' => false, 'complete_progress' => true, 'sqs_data' => $sqsData];
    }

    public function requestStatus($remoteShopId, $sqsData)
    {
        $filter['shop_id'] = $remoteShopId;
        $filter['id'] = $sqsData['data']['id'];
        //$filter['save_file'] = 1;
        $remoteResponse = $this->_apiClient->init($this->_target, 'true')->call('/bulk/operation', [], $filter, 'GET');

        $this->di->getLog()->logContent('PROCESS 0020 | Vistar Import | Class - Import | Function Name - requestStatus | Response from Remote Server = ' . print_r($remoteResponse, true), 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'product' . DS . date("Y-m-d") . DS . 'vistar_product_import.log');

        if (!isset($remoteResponse['success']) && !$remoteResponse['success']) return ['success' => false, 'requeue' => false, 'complete_progress' => true, 'sqs_data' => $sqsData];

        if (($remoteResponse['data']['status'] == 'RUNNING') || ($remoteResponse['data']['status'] == 'CREATED')) {
            $sqsData['data']['retry'] = isset($sqsData['data']['retry']) ? (int)($sqsData['data']['retry'] + 1) : 1;

            // ** START ** Code for Handling Continuously status RUNNING with same object count
            /*if($remoteResponse['data']['status'] == 'RUNNING'){
                if(isset($sqsData['data']['objectCountArray']) && count($sqsData['data']['objectCountArray'])!=0){
                    $abc=$sqsData['data']['objectCountArray'];
                    array_push($abc,$remoteResponse['data']['objectCount']);
                    $sqsData['data']['objectCountArray']=$abc;
                }else{
                    $sqsData['data']['objectCountArray']= [$remoteResponse['data']['objectCount']];
                }
                if(isset($sqsData['data']['objectCountArray'])){
                    $this->di->getLog()->logContent('Retry count = ' . $sqsData['data']['retry']. ' | objectCountArray Array = '.print_r($abc,true), 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'product' . DS . date("Y-m-d") . DS . 'vistar_product_import.log');
                    $currentobj_count=end($sqsData['data']['objectCountArray']);
                    $indexes = array_keys($sqsData['data']['objectCountArray'], $currentobj_count);
                    if(count($indexes)>6){
                        $queuedTask = new \App\Core\Models\QueuedTask;
                        $queuedTask->deleteFailedQueuetask($sqsData['data']['feed_id']);
                        $cancelBulkOperation = $this->di->getObjectManager()->get("\App\Connector\Components\ApiClient")
                            ->init('shopify', 'true')
                            ->call("/bulk/operation", [], ['shop_id' => $remoteShopId, 'id' => $remoteResponse['data']['id']], 'DELETE');

                        $this->di->getLog()->logContent('PROCESS 00201 | Import | requestStatus | Bulk product import process Failed Now Importing Product with Rest API '.json_encode($cancelBulkOperation), 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'product' . DS . date("Y-m-d") . DS . 'vistar_product_import.log');
                        $handlerData = $this->di->getObjectManager()->get("\App\Shopifyhome\Components\Product\Helper")->shapeSqsData($this->di->getUser()->id, $remoteShopId);
                        $this->di->getObjectManager()->get("\App\Shopifyhome\Components\Product\Route\Requestcontrol")->fetchShopifyProduct($handlerData);
                        return true;
                    }
                }
            }*/
            // ** END ** Code for Handling Continuously status RUNNING with same object count

            $this->di->getLog()->logContent('PROCESS 0021 | Vistar Import | Import | requestStatus | retry count = ' . $sqsData['data']['retry']. ' | Currently Processed : '.$remoteResponse['data']['objectCount']. ' objects.', 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'product' . DS . date("Y-m-d") . DS . 'vistar_product_import.log');

            $msg = Import::SHOPIFY_QUEUE_PRODUCT_IMPORT_MSG . ' : Shopify Processing Your Request. Currently Processed : ' . $remoteResponse['data']['objectCount']. ' objects.';
            $progress = $this->_connectorHelper->updateFeedProgress($sqsData['data']['feed_id'], $sqsData['data']['individual_weight'], $msg);
            $sqsData['data']['individual_weight'] = 0;
            return ['success' => true, 'requeue' => true, 'sqs_data' => $sqsData];

        } elseif ($remoteResponse['data']['status'] == 'COMPLETED'){
            if(strlen($remoteResponse['data']['url']) == 0) return ['success' => false, 'requeue' => false, 'sqs_data' => $sqsData];

            if($sqsData['data']['individual_weight'] == 0) $currentWt = 3; else $currentWt = 4;
            $isFileSaved = $this->_vistarHelper->saveFile($remoteResponse['data']['url']);
            if (!$isFileSaved['success']) {
                $this->di->getLog()->logContent('PROCESS 00201 | Import | requestStatus | ERROR | Response  = ' . print_r($isFileSaved, true), 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'product' . DS . date("Y-m-d") . DS . 'vistar_product_import.log');

                return ['success' => false, 'requeue' => false, 'sqs_data' => $sqsData];
            }
            unset($sqsData['data']['retry']);
            $sqsData['data']['operation'] = 'build_progress';

            /*$filePath = BP . DS . 'var' . DS . 'file' . DS . '1' . DS . 'product.jsonl';
            $sqsData['data']['file_path'] = $filePath;*/
            $sqsData['data']['file_path'] = $isFileSaved['file_path'];

            $sqsData['data']['individual_weight'] = 3;
            $msg = Import::SHOPIFY_QUEUE_PRODUCT_IMPORT_MSG . ' : Your Product(s) import Request is successfully processed by Shopify.';
            $progress = $this->_connectorHelper->updateFeedProgress($sqsData['data']['feed_id'], $currentWt, $msg);
            return ['success' => true, 'requeue' => true, 'sqs_data' => $sqsData];

        } else {

            $this->di->getLog()->logContent('PROCESS 0021 | Import | requestStatus | ERROR | Response  = ' . print_r($remoteResponse, true), 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'product' . DS . date("Y-m-d") . DS . 'vistar_product_import.log');
            return ['success' => false, 'requeue' => false, 'sqs_data' => $sqsData];
        }
    }

    public function buildProgressBar($sqsData)
    {
        $filePath = $sqsData['data']['file_path'];
        try {
            $remoteResponse = $this->di->getObjectManager()->create("\App\Shopifyhome\Components\Parser\JSONLParser", [$filePath])->getItemCount();
            $remoteResponse['data'] = $remoteResponse['data']['ProductVariant'];
        } catch (\Exception $e) {
            $this->di->getLog()->logContent('PROCESS 00030 | Process - Get Product Count from Shopify | Import | buildProgressBar | Error Message  = ' . $e->getMessage() . ' File Path = ' . $filePath . ' | Probable Error : JSONL file does not exist.', 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'product' . DS . date("Y-m-d") . DS . 'vistar_product_import.log');
            return ['success' => false, 'requeue' => false, 'complete_progress' => true, 'sqs_data' => $sqsData];
        }

        $this->di->getLog()->logContent('PROCESS 0030 | Vistar Import | buildProgressBar |Process - Get Product Count from Shopify | Response from Remote Server = \n' . print_r($remoteResponse, true), 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'product' . DS . date("Y-m-d") . DS . 'vistar_product_import.log');

        if (!isset($remoteResponse['success']) || !$remoteResponse['success']) return ['success' => false, 'requeue' => false, 'complete_progress' => true, 'sqs_data' => $sqsData];

        if ($remoteResponse['data'] == 0) return ['success' => false, 'requeue' => false, 'complete_progress' => true, 'notice' => 'No Product Found on your Shopify Store', 'sqs_data' => $sqsData];
        else {
            $sqsData['data']['total'] = $remoteResponse['data'];
            $sqsData['data']['operation'] = 'import_product';
            $sqsData['data']['limit'] = Import::PAGE_SIZE;
            $sqsData['data']['total_variant'] = 0;

            $msg = Import::SHOPIFY_QUEUE_PRODUCT_IMPORT_MSG . " : {$remoteResponse['data']} Variant(s) found on your Shopify store.";
            $progress = $this->_connectorHelper->updateFeedProgress($sqsData['data']['feed_id'], $sqsData['data']['individual_weight'], $msg);

             /*unset($sqsData['data']['operation']);*/

            return ['success' => true, 'requeue' => true, 'sqs_data' => $sqsData];
        }
    }

    public function importControl($sqsData){

        $filter['limit'] = $sqsData['data']['limit'];
        $totalTime = $totalVariant = 0;
        $returnParam = [];
        $this->di->getLog()->logContent('', 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'product' . DS . date("Y-m-d") . DS . 'system.log');
        do{
            try{

                $openTime = microtime(true);

                if (isset($sqsData['data']['next'])) $filter['next'] = $sqsData['data']['next'];
                $intermediateResponse = $this->di->getObjectManager()->create("\App\Bigcommercehome\Components\Parser\Product", [$sqsData['data']['file_path']]);
                $response =  $intermediateResponse->getVariants($filter);
                $remote = $this->_vistarHelper->shapeProductResponse($response);
               
                $operationStats = $this->pushToProductContainer($remote, 'vistar');

                if(!$operationStats['success']) return [ 'success' => false, 'requeue'=> false , 'complete_progress' => true, 'notice' => $operationStats['message'], 'sqs_data' => $sqsData ];

                $totalVariant = $sqsData['data']['total_variant'] + $remote['count']['variant'];
                $sqsData['data']['total_variant'] = $totalVariant;
                $sqsData['data']['next'] = $remote['cursors']['next'] ?? '';
                $this->_vistarUtility->updateFeedProgress($remote['count']['variant'], $sqsData, Import::IMPORT_FEED_WEIGHT);
                
                $endTime = microtime(true);     $elapsedtIME = $endTime - $openTime;     $totalTime += $elapsedtIME;
                $generalizedTime = $this->di->getObjectManager()->get('\App\Shopifyhome\Components\Utility')->secondsToTime($elapsedtIME);

                $this->di->getLog()->logContent('Variant Import Count : '.$sqsData['data']['total_variant'].' | Time Taken : '.$generalizedTime , 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'product' . DS . date("Y-m-d") . DS . 'system.log');
                $this->di->getLog()->logContent('Total Current Loop Time : '.$totalTime , 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'product' . DS . date("Y-m-d") . DS . 'system.log');

                $cutOff = $this->_vistarUtility->canTerminateExecution($elapsedtIME, $totalTime, $sqsData['data']['sqs_timeout'], $sqsData['data']['next']);

                $this->di->getLog()->logContent('CutOff '.json_encode($cutOff).' | Next Cursor : '. $sqsData['data']['next'], 'info', 'shopify' . DS . $this->di->getUser()->id . DS . 'product' . DS . date("Y-m-d") . DS . 'system.log');

                if($cutOff && !empty($sqsData['data']['next'])) $returnParam = ['success' => true, 'requeue' => true, 'sqs_data' => $sqsData, 'data' => $operationStats['stats']];
                elseif (empty($sqsData['data']['next'])){
                    $sqsData['data']['operation'] = 'delete_nexisting';
                    unset($sqsData['data']['file_path'], $sqsData['data']['total_variant'], $sqsData['data']['limit'], $sqsData['data']['sqs_timeout'], $sqsData['data']['next']);
                    $returnParam = ['success' => true, 'requeue' => false, 'is_complete' => true, 'sqs_data' => $sqsData, 'data' => $operationStats['stats']];
                }
            } catch (\Exception $e){
                $this->di->getLog()->logContent('PROCESS X00400 | Process - Get Product Count from Shopify | Bulkimport | buildProgressBar | Error Message  = '.$e->getMessage().' File Path = '. $sqsData['data']['file_path'].' | Probable Error : JSONL file does not exist.','info','shopify'.DS.$this->di->getUser()->id.DS.'product'.DS.date("Y-m-d").DS.'vistar_product_import.log');
                return [ 'success' => false, 'requeue'=> false , 'complete_progress' => true, 'sqs_data' => $sqsData ];
            }

        } while (!$cutOff);

        return $returnParam;
    }

    public function pushToProductContainer($productData, $mode = '')
    {
        $helper = $this->di->getObjectManager()->get('\App\Shopifyhome\Components\Product\Helper');
        $productContainer = $this->di->getObjectManager()->create('\App\Connector\Models\ProductContainer');
        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $mongo->setSource("product_container");
        $mongoCollection = $mongo->getPhpCollection();


        $this->di->getLog()->logContent(' PROCESS 003002 | Import | pushToProductContainer | Inside pushToProductContainer ','info','shopify'.DS.$this->di->getUser()->id.DS.'product'.DS.date("Y-m-d").DS.$mode.'_product_import.log');

        $simpleProductCounter = 0;

        $bulkOpArray = [];

        foreach ($productData['data'] as $productData){
             //$exists = $productContainer->getProduct(['source_product_id' => (string)$productData['container_id']]);
            $exists = $mongoCollection->findOne(['source_product_id' => (string)$productData['id']]);
            
            if(!empty($exists)){
                $productData['marketplace'] = $this->di->getObjectManager()->get('\App\Shopifyhome\Components\Shop\Shop')->getUserMarkeplace();
                $productData['db_action'] = "variant_update";
                $data = $helper->formatCoreDataForVariantData($productData);
                $query = $this->di->getObjectManager()->get('\App\Shopifyhome\Components\Product\Helper')->getAndQuery(['source_product_id' => $data['source_product_id']],$this->di->getUser()->id);
                $bulkOpArray[] = [
                    'updateOne' => [
                        $query,
                        [ '$set' => $data]
                    ]
                ];
            } else {
                $data['details'] = $helper->formatCoreDataForDetail($productData);
                $data['variant_attribute'] = isset($productData['variant_attributes']) ? $productData['variant_attributes'] : [];
                $data['variants'][] = $helper->formatCoreDataForVariantData($productData);
                $target = $this->di->getObjectManager()->get('\App\Shopifyhome\Components\Shop\Shop')->getUserMarkeplace();
                $productContainer->setSource("product_container");
                $res = $productContainer->createProductsAndAttributes([$data], $target, 0, $this->di->getUser()->id);
                if(isset($res[0]['success']) && $res[0]['success']) $simpleProductCounter++;
                $this->di->getLog()->logContent(' PROCESS 00310 | Import | pushToProductContainer | Product Import REsponse : '.json_encode($res),'info','shopify'.DS.$this->di->getUser()->id.DS.'product'.DS.date("Y-m-d").DS.$mode.'_product_import.log');
            }
            $data = [];
        } try{
            if(!empty($bulkOpArray)){
                $bulkObj = $mongoCollection->BulkWrite($bulkOpArray, ['w' => 1]);
                $returenRes = [
                    'acknowledged' => $bulkObj->isAcknowledged(),
                    'inserted' => $bulkObj->getInsertedCount(),
                    'modified' => $bulkObj->getModifiedCount(),
                    'matched' => $bulkObj->getMatchedCount(),
                    'mainProductCount' => $simpleProductCounter
                ];
                $this->di->getLog()->logContent(' PROCESS 00310 | Import | pushToProductContainer | Bulk Product Write Response : '.print_r($returenRes, true),'info','shopify'.DS.$this->di->getUser()->id.DS.'product'.DS.date("Y-m-d").DS.$mode.'_product_import.log');
                return [
                    'success' => true,
                    'stats' => $returenRes
                ];
            }
        } catch (\Exception $e) {
            return ['success' => false, 'message' => $e->getMessage()];
        }
        return [
            'success' => true,
            'stats' => $simpleProductCounter. ' main product imported successfully'
        ];
    }
}