<?php
/**
 * CedCommerce
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the End User License Agreement (EULA)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://cedcommerce.com/license-agreement.txt
 *
 * @category    Ced
 * @package     Ced_Shopifyhome
 * @author      CedCommerce Core Team <connect@cedcommerce.com>
 * @copyright   Copyright CEDCOMMERCE (http://cedcommerce.com/)
 * @license     http://cedcommerce.com/license-agreement.txt
 */
namespace App\Bigcommercehome\Components\Shop;

use App\Bigcommercehome\Components\Core\Common;
use Exception;

class Shop extends Common
{
    const UNIQUE_SHOP_IDENTIFIER = 'id';

    const UNIQUE_WAREHOUSE_IDENTIFIER = 'id';

    const MARKETPLACE = 'bigcommerce';

    public function addShopToUser($data = [], $remoteshopId = 0)
    {
        if(empty($data)){
            $target = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Shop')->getUserMarkeplace();
            $data = $this->di->getObjectManager()->get('\App\Connector\Components\ApiClient')->init($target, true)->call('/shop', [], [
                'shop_id' => $remoteshopId
            ]);
            if(isset($data['errors']) || is_null($data) || empty($data)){
                return [
                    'success' => false,
                    'message' => isset($data['errors']) ? $data['errors'] : 'Error fetching data from Bigcommerce, Please try again later or contact our next available support member.'
                ];
            }
        }
        $shopData = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Utility')->shopNodeToExcludeAndFormatData($data['data']);
        if($shopData['marketplace']=="Bigcommerce"){
            $remoteResponse = $this->di->getObjectManager()->get("\App\Connector\Components\ApiClient")
            ->init('bigcommerce', 'true')
            ->call('/savehomeadminid',[],['shop_id'=> $data['data']['remote_shop_id'] , 'admin_home_id' =>$this->di->getUser()->id], 'POST');
        }

        if(!isset($shopData['marketplace'])) $shopData['marketplace'] = Shop::MARKETPLACE;
        if(isset($data['channel_id'])){
            $shopData['channel_id'] = $data['channel_id'];
        }

        $shopCollection = $this->di->getObjectManager()->create('\App\Core\Models\User\Details');
        $shopRes = $shopCollection->addShop($shopData, false, [Shop::UNIQUE_SHOP_IDENTIFIER]);
        if($shopRes['success']){
            return [
                'success' => true,
                'home_internal_shop' => $shopRes['data']['shop_id'],
                'message' => 'New Shop has been successfully added.'
            ];
        } else return $shopRes;

    }

    /*
        $remoteshopId : not required during initial authentication
    */
    public function addWarehouseToUser( $homeShopId, $data, $target = 'shopify', $customStoreData){
        $addACustomWarehouseforBigCommerce = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Shop')->addACustomWarehouseforBigCommerce($customStoreData);
        $locationResponse['data'] = [$addACustomWarehouseforBigCommerce];

        $warehouseCollection = $this->di->getObjectManager()->create('\App\Core\Models\User\Details');
        $sCount = $eCount = 0;
        foreach($locationResponse['data'] as $warehouse){
            $warehouse = $this->di->getObjectManager()->get('\App\Shopifyhome\Components\Shop\Utility')->locationNodeToExcludeAndFormatData($warehouse);
            $shopRes = $warehouseCollection->addWarehouse($homeShopId, $warehouse, false, [Shop::UNIQUE_WAREHOUSE_IDENTIFIER]);
            $this->di->getLog()->logContent('message :'.  $sCount,'info','user_login.log');

            if($shopRes['success']) $sCount++;
            else $eCount++;
        }
        if($sCount > 0) {
            $this->di->getLog()->logContent('message :'.  $sCount. ' new warehouse has been successfully added.','info','user_login.log');
            return ['success' => true, 'message' => $sCount. ' new warehouse has been successfully added.'];


        }
        else {
            $this->di->getLog()->logContent('message'. $sCount. 'Error adding new warehouse.','info','user_login.log');
            return ['success' => false, 'message' => 'Error adding new warehouse.'];
        }
    }

    public function getUserMarkeplace()
    {
        return "bigcommerce";
        $mId = (string)$this->di->getUser()->id;
        // $mId=(string)"6076abf25bce6f1fb84fe6c4";
        //  print_r($mId);die("");
        $mongoCollection = $this->di->getObjectManager()->get('\App\Core\Models\BaseMongo');
        $mongoCollection->setSource('user_details');
        $phpMongoCollection = $mongoCollection->getPhpCollection();
// print_r($phpMongoCollection);die;
        $options = [];

        $pipeline = [
            [
                '$match' => [
                    "user_id" => $mId
                ]
            ],
            [
                '$project' => [
                    'marketplace' => [
                        '$arrayElemAt' => [
                            '$shops.marketplace',
                            0
                        ]
                    ]
                ]
            ]
        ];
        $userData = $phpMongoCollection->aggregate($pipeline, $options)->toArray();
        // print_r($userData);die;
        if(isset($userData[0]['marketplace'])){
            return $userData[0]['marketplace'];
        }
    }

    public function matchMarketplaceWithRemoteSubAppId($remoteShopId)
    {
        $marketplace[$this->di->getConfig()->apiconnector->shopify->sub_app_id] = 'shopify';
        $marketplace[$this->di->getConfig()->apiconnector->shopify_ig->sub_app_id] = 'shopify_ig';

        $endMarket =  $marketplace[$remoteShopId];
        return ($endMarket == 'shopify_ig') ? 'shopify_ig' : 'shopify';
    }

    public function getShopDetailByUserID($user_id = false, $marketplace = false)
    {
        if ( !$user_id ) $user_id = $this->di->getUser()->id;

        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $collection = $mongo->setSource("user_details")->getCollection();
        $user_details = $collection->findOne(["user_id"=>$user_id]);

        if ( empty($user_details) ) return ['success' => false, 'message' => 'Shop details not found'];

        if ( $marketplace ) {
            foreach ( $user_details['shops'] as $value ) {
                if ( $value['marketplace'] === $marketplace ) {
                    return $value;
                }
            }
        }
        return [];
    }

    /* As BigCommerce doesn't comes with Multiple Location feature */
    public function addACustomWarehouseforBigCommerce($shopDetails){
        return [
            "id" => $shopDetails['store_id'],
            "name" => $shopDetails['address'],
            "address1" => $shopDetails['address'],
            "address2" => $shopDetails['address'],
            "city" => $shopDetails['address'],
            "zip" => $shopDetails['default_channel_id'],
            "channel_id" => $shopDetails['default_channel_id'],
            "province" => $shopDetails['address'],
            "country" => $shopDetails['country'],
            "phone" => $shopDetails['phone'],
            "country_code" => $shopDetails['country_code'],
            "country_name" => $shopDetails['country'],
            "province_code" => $shopDetails['address'],
            "active" => true,
            "localized_country_name" => $shopDetails['country'],
            "localized_province_name" => $shopDetails['address']
        ];
    }
}