<?php
/**
 * CedCommerce
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the End User License Agreement (EULA)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://cedcommerce.com/license-agreement.txt
 *
 * @category    Ced
 * @package     Ced_Shoifyhome
 * @author      CedCommerce Core Team <connect@cedcommerce.com>
 * @copyright   Copyright CEDCOMMERCE (http://cedcommerce.com/)
 * @license     http://cedcommerce.com/license-agreement.txt
 */


namespace App\Bigcommercehome\Models;

use App\Core\Components\ObjectManager;
use App\Core\Models\SourceModel as BaseSourceModel;
use OAuth\OAuth2\Service\Mondo;

class SourceModel extends BaseSourceModel
{
    const BIGCOMMERCE_QUEUE_PRODUCT_IMPORT_MSG = 'BIGCOMMERCE_PRODUCT_IMPORT';

    const BIGCOMMERCE_QUEUE_INV_IMPORT_MSG = 'BIGCOMMERCE_INVENTORY_SYNC';

    const BIGCOMMERCE_QUEUE_ORDER_IMPORT_MSG = 'BIGCOMMERCE_ORDER_IMPORT';
    const BIGCOMMERCE_QUEUE_CUSTOMER_IMPORT_MSG = 'BIGCOMMERCE_CUSTOMER_IMPORT';
    const BIGCOMMERCE_QUEUE_CART_IMPORT_MSG = 'BIGCOMMERECE_ABANDONED_CART_IMPORT';

    const Max_size=250;

    public function commenceHomeAuth($remotePostData)
    {
        $validateAndCreateUser = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Authenticate\User')->setupHomeUser($remotePostData);
        $appCode = isset($remotePostData['rawResponse']['app_code']) ? $remotePostData['rawResponse']['app_code'] : 'default';
        $validateAndCreateUser['redirect_to_dashboard'] = false;
        if ($validateAndCreateUser['success'] && !isset($validateAndCreateUser['redirect_to_dashboard'])) {
            $validateAndCreateUser['heading'] = 'Congratulations!!';
            $validateAndCreateUser['message'] = 'Welcome onboard. Your BigCommerce store has been succesfully set up.';
            if (isset($validateAndCreateUser['mail'])) {
                $this->di->getObjectManager()->get("\App\Bigcommercehome\Components\Mail\Seller")->onboard($validateAndCreateUser['mail']);
            }
            try {
                $this->registerUninstallWebhook($appCode);
            } catch (\Exception $e) {
                $this->di->getLog()->logContent('error registering uninstall webhook for user id : ' . $this->di->getUser()->id, 'info', 'uninstallWebhook.log');
            }

            return $validateAndCreateUser;
        } else {
            return $validateAndCreateUser;
        }
    }


    public function initiateUpload($marketplaceDetails, $userId = false)
    {
        if (!$userId) {
            $userId = $this->di->getUser()->id;
        }
        $source = $marketplaceDetails['source'];
        $sourceShopId = $marketplaceDetails['source_shop_id'];
        $targetShopId = $marketplaceDetails['target_shop_id'];
        $profileId = $marketplaceDetails['selected_profile']; //not come
        $prodCount = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Data')->getProductsCountByQuery($userId, $profileId, $source);
        $this->di->getLog()->logContent(' Total Product Count and Uploaded Product Count == '.print_r($prodCount, true), 'info', 'bigcommerce'.DS.$userId.DS.'product_upload'.DS.date("Y-m-d").DS.'product_upload.log');
        $finalCount = 0;
        if ($prodCount) {
            $finalCount = $prodCount['product_count'] + $prodCount['product_uploaded_count'];
        }
        if ($finalCount < 1) {
            return ['success' => false, 'message' => 'No products found for upload. Kindly import products to app then upload them to BigCommerce.'];
        }
        $productCount = $prodCount['product_count'];
        $productUploadedCount = $prodCount['product_uploaded_count'];

        $sync_status='enable';

        if ($this->di->getConfig()->enable_rabbitmq_internal) {
            $queuedTask = new \App\Connector\Models\QueuedTasks;
            $msg = "Uploading";
            $msgForSync = "Product Sync Process from BigCommerce";
            $bigcommerceConfigData = $this->di->getObjectManager()->get('\App\Connector\Models\QueuedTasks');
            $sellerData = $bigcommerceConfigData::query()->where("user_id = $userId")->andWhere("shop_id = $targetShopId")->andWhere("message LIKE '%$msg%'")->execute()->toArray();
            $productSyncData = $bigcommerceConfigData::query()->where("user_id = $userId")->andWhere("message LIKE '%$msgForSync%'")->execute()->toArray();
            if (count($sellerData) || count($productSyncData)) {
                return ['success' => false, 'message' => 'Currently either product syncing or upload process is already under progress. Please check notification for updates.'];
            }
            $maxLimit = 10;
            if ($maxLimit > $productCount) {
                $maxLimit = $productCount + $productUploadedCount;
            } else {
                $maxLimit = $maxLimit + $productUploadedCount;
            }

            $this->di->getLog()->logContent(' MAX LIMIT == '.\GuzzleHttp\json_encode($maxLimit), 'info', 'bigcommerce'.DS.$userId.DS.'product_upload'.DS.date("Y-m-d").DS.'product_upload.log');

            if ($productCount == 0 &&
                $sync_status == 'disable') {
                return ['success' => false, 'message' => 'product syncing is disabled. You can change it in the Settings section.'];
            }
            if ($maxLimit > 0) {
                $productMessage = 'products';
                if ($productCount == 1) {
                    $productMessage = 'product';
                }
                $queuedTask->set([
                    'user_id' => $userId,
                    'message' => 'Uploading ' . $productCount . ' ' . $productMessage . ' to BigCommerce',
                    'progress' => 0.00,
                    'shop_id' => $targetShopId
                ]);
                $queuedTask->save();
                $fileName = 'failed_product_' . $userId . '_' . time();
                $filePath = BP . DS . 'var' . DS . 'failed-products' . DS . $fileName . '.php';
                $data['data'] = [
                    'page' => 1,
                    'max_limit' => $productCount,
                    'user_id' => $userId,
                    'source' => $source,
                    'feed_id' => $queuedTask->id,
                    'cursor' => 0,
                    'profileId' => $profileId,
                    'source_shop_id' => $sourceShopId,
                    'target_shop_id' => $targetShopId,
                    'total_count' => $productCount,
                    'uploaded_product_count' => $productUploadedCount,
                    'sync_status' => $sync_status,
                    'file_path' => $filePath,
                    'file_name' => $fileName,
                    'success_message' => 'All selected products successfully uploaded to BigCommerce',
                    'failure_message' => 'Failed to upload all products to BigCommerce'
                ];

                $this->di->getLog()->logContent(' initProductUploadQueue DATA  == '.print_r($data['data'], true), 'info', 'bigcommerce'.DS.$userId.DS.'product_upload'.DS.date("Y-m-d").DS.'DATA.log');
                
                $response = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Data')->initProductUploadQueue($data, $failed_product = false);
                if (isset($response['success']) && ($response['success'] == false)) {
                    return ['success' => false, 'message' => $response['message']];
                }
                return ['success' => true, 'code' => 'product_upload_started', 'message' => 'product upload process started from app to bigcommerce.'];
            }
            return ['success' => false, 'message' => 'You have exhausted your limit of uploading products on BigCommerce. To upload more products kindly buy upload credits to upload more products.', 'code' => 'limit_exhausted'];
        } else {
            return ['success' => false, 'message' => 'RMQ_DISABLE : Internal Server Error . We are working hard and will be up shortly'];
        }
    }

    public function updateProduct($updateData, $userId = false)
    {
        return $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Data')->handleProductData($updateData, $userId);
    }

    public function getsellers($marketplaceDetails = [], $userId = null)
    {
        return $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Vendor\Vendor')->getvendors($marketplaceDetails, $userId);
    }
    public function getProductHelper()
    {
        return $this->di->getObjectManager()->get("\App\Bigcommercehome\Components\Product\Vistar\Helper");
    }
    
    public function initiateImport($data = [], $userId = null, $force = false):array
    {
        $userId = $this->di->getUser()->id;
        if (!$userId) {
            return [ 'success' => false, 'message' => 'Invalid User. Please check your login credentials' ];
        }

        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $collection = $mongo->getCollectionForTable('queued_tasks');
        if ( $collection->count([
            'user_id' => $this->di->getUser()->id,
            // 'code' => 'mercado_cbt_upload',
        ]) > 0 ) {
            return [
                'errorFlag' => true,
                'message' => 'process is already under progress. Please check activities for updates.',
                'code' => 'already_in_progress',
            ];
        }
        
        $target = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Shop')->getUserMarkeplace();
        
        $userData = $this->di->getObjectManager()->get("\App\Core\Models\User\Details")->getDataByUserID($userId, $target);
        if (isset($userData['message'])) {
            return $userData;
        }
        
        $remoteShopId = $userData->remote_shop_id;
        $domain = $userData->domain;
        
        // $queueData = [
        //     'user_id' => $userId,
        //     'message' =>'initializing Product Import...',
        //     'type' => 'bigcommerce_product_import',
        //     'progress' => 0.00,
        //     'shop_id' => (string) $userData['_id']
        // ];
        // $queuedTask = new \App\Connector\Models\QueuedTasks;
        // $queuedTaskId = $queuedTask->setQueuedTask($userId, $queueData);

        $obj = $this->di->getObjectManager()->get('\App\Connector\Components\Helper');
        $feed_id = $obj->setQueuedTasks('Please wait while product(s) details are being import...', 'bigcommerce_product_import');

        if ( isset($feed_id['errorFlag']) ) {
            return ['success' => false, 'message' => 'Product Import process is already under progress. Please check notification for updates.'];
        } else {
            $handlerData = $this->di->getObjectManager()->get("\App\Bigcommercehome\Components\Product\Helper")->shapeSqsData($userId, $remoteShopId, $feed_id);
            if (isset($userData->app_code)) {
                $handlerData['data']['app_code'] = $userData->app_code;
            }
            
            $rmqHelper = $this->di->getObjectManager()->get('\App\Rmq\Components\Helper');
            return [
                'success' => true,
                'message' => 'BigCommerce Product Import Initiated',
                'queue_sr_no' => $rmqHelper->createQueue($handlerData['queue_name'], $handlerData)
            ];
        }
    }
 
    public function initiateInvImport($marketplaceDetails = [], $userId = null)
    {
        $target = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Shop')->getUserMarkeplace();
        $userId = $userId ?? $this->di->getUser()->id;
        // print_r($userId);die();
        if (!$userId) {
            return [
                'success' => false,
                'message' => 'Invalid User. Please check your login credentials'
            ];
        }
        $remoteShopId = $this->di->getObjectManager()->get("\App\Core\Models\User\Details")->getDataByUserID($userId, $target)->remote_shop_id;
        // print_r($remoteShopId);die;
        /*
         * todo : add shop_id check in phase-2 development
         * */
        $sMsg = SourceModel::BIGCOMMERCE_QUEUE_PRODUCT_IMPORT_MSG;
        $iMsg = SourceModel::BIGCOMMERCE_QUEUE_INV_IMPORT_MSG;
        // $bigcommerceConfigData = $this->di->getObjectManager()->get('\App\Connector\Models\QueuedTasks');
        // $sellerData = $bigcommerceConfigData::query()->where("user_id = $userId")->andWhere("message LIKE '%$sMsg%' OR message LIKE '%$iMsg%'")->execute()->toArray();
        if (!empty($sellerData)) {
            return ['errorFlag' => true, 'msg' => 'Product Import process OR Location Sync is already under progress. Please check notification for updates.'];
        }

        try {
            $queuedTask = new \App\Connector\Models\QueuedTasks;
            $queuedTask->set([
                'user_id' => (int)$userId,
                'message' => SourceModel::BIGCOMMERCE_QUEUE_INV_IMPORT_MSG . ' : Please Wait while locations from BigCommerce is being synced',
                'progress' => 0.00,
                'shop_id' => (int)$userId
            ]);
            $status = $queuedTask->save();
        } catch (\Exception $e) {
            echo $e->getMessage();
        }

        $handlerData = [
            'type' => 'full_class',
            'class_name' => '\App\Bigcommercehome\Components\Product\Route\Requestcontrol',
            'method' => 'fetchProductInventoryLocationWise',
            'queue_name' => 'bigcommerce_product_inv_import',
            'own_weight' => 100,
            'user_id' => $userId,
            'data' => [
                'user_id' => $userId,
                'individual_weight' => 0,
                'pageSize' => 40,
                'feed_id' => ($status ? $queuedTask->id : false),
                'remote_shop_id' => $remoteShopId,
                'cursor' => 0
            ]
        ];
        // $this->di->getLog()->logContent('','info','bigcommerce'.DS.$this->di->getUser()->id.DS.'inventory'.DS.date("Y-m-d").DS.'inventory.log');
        // $this->di->getLog()->logContent('************************************************************************************************','info','bigcommerce'.DS.$this->di->getUser()->id.DS.'inventory'.DS.date("Y-m-d").DS.'inventory.log');
        $rmqHelper = $this->di->getObjectManager()->get('\App\Rmq\Components\Helper');
        //   print_r($rmqHelper->createQueue($handlerData['queue_name'], $handlerData));die;
  
        //$this->di->getObjectManager()->get("App\Bigcommercehome\Components\Product\Inventory\Data")->deleteLocations();
        return [
            'success' => true,
            'message' => 'Bigcommerce Product Inventory Import Initiated',
            'queue_sr_no' => $rmqHelper->createQueue($handlerData['queue_name'], $handlerData)
        ];
    }

    public function saveDetails($data, $userId = null)
    {
        $userId = $userId ?? $this->di->getUser()->id;
        if (isset($data['username']) && isset($data['email']) && isset($data['password']) && isset($data['mobno'])) {
            $userDetails = $this->di->getObjectManager()->create('\App\Core\Models\User\Details');
            $collection = $userDetails->getCollection();
            $details=[
                'username'=>$data['username'],
                'email'=>$data['email'],
                'marketplace'=>'cedcommerce'
            ];
            $details['phone']=$data['mobno'];
            $collection->updateOne(
                ['user_id'=>$userId],
                ['$set'=>['shops'=>[$details]]]
            );
            return ['success'=>true];
        }
    }

    public function getAttributes($category = '')
    {
        $attributes = [
            [
                'code' => 'title',
                'required' => 1,
                'title' => 'Title',
                'system' => 1,
                'visible' => 0
            ],
            [
                'code' => 'sku',
                'required' => 1,
                'title' => 'Sku',
                'system' => 1,
                'visible' => 0
            ],
            // [
            //     'code' => 'price',
            //     'required' => 1,
            //     'title' => 'Price',
            //     'system' => 1,
            //     'visible' => 0
            // ],
            // [
            //     'code' => 'short_description',
            //     'required' => 1,
            //     'title' => 'Description',
            //     'system' => 1,
            //     'visible' => 0
            // ],
            // [
            //     'code' => 'product_type',
            //     'required' => 1,
            //     'title' => 'Product Type',
            //     'system' => 1,
            //     'visible' => 0
            // ],
            // [
            //     'code' => 'vendor',
            //     'required' => 1,
            //     'title' => 'Vendor',
            //     'system' => 1,
            //     'visible' => 0
            // ],
            // [
            //     'code' => 'tags',
            //     'required' => 1,
            //     'title' => 'Tag',
            //     'system' => 1,
            //     'visible' => 0
            // ],
            // [
            //     'code' => 'quantity',
            //     'required' => 1,
            //     'title' => 'Quantity',
            //     'system' => 1,
            //     'visible' => 0
            // ],
            // [
            //     'code' => 'source_variant_id',
            //     'required' => 1,
            //     'title' => 'Variant Id',
            //     'system' => 1,
            //    'visible' => 0
            // ],
            // [
            //     'code' => 'source_product_id',
            //     'required' => 1,
            //     'title' => 'Parent Id',
            //     'system' => 1,
            //     'visible' => 0
            // ]
        ];
        $collections = $this->getCollectionsOptions();
        if ($collections &&
             count($collections)) {
            $attributes[] = [
                 'code' => 'brand',
                 'required' => 1,
                 'title' => 'Brand',
                 'system' => 1,
                 'visible' => 0,
                 'options' => $collections
             ];
        }

        $collections = $this->getProductCategoriesOptions();
        if ($collections &&
             count($collections)) {
            $attributes[] = [
                 'code' => 'collection.value',
                 'required' => 1,
                 'title' => 'Category',
                 'system' => 1,
                 'visible' => 0,
                 'options' => $collections
             ];
        }
        return $attributes;
    }
    public function getCollectionsOptions()
    {
        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $collection = $mongo->getCollectionForTable('product_container');
        $aggregate = [];
        $aggregate[] = [ '$match' => [ "user_id" => (string)$this->di->getUser()->id]];
        // $aggregate[] = ['$unwind' => '$variants'];
        // $aggregate[] = ['$unwind' => '$variants.collection'];
        $aggregate[] = ['$group' => [
            '_id' => '$brand',
        ]];


        $collection = $collection->aggregate($aggregate)->toArray();

        $options = [];


        foreach ($collection as $value) {
            $options[] = ['label' => $value['_id'],'value' => $value['_id'],];
        }
        return $options;
    }
    public function getProductCategoriesOptions()
    {
        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $collection = $mongo->getCollectionForTable('product_container');
        $aggregate = [];
        $aggregate[] = [ '$match' => [ "user_id" => (string)$this->di->getUser()->id]];
        $aggregate[] = [ '$unwind' => '$collection'];
        $aggregate[] = ['$group' => ['_id' => '$collection.value']];


        $collection = $collection->aggregate($aggregate)->toArray();

        $options = [];

        foreach ($collection as $value) {
            $options[] = ['label' => $value['_id'],'value' => $value['_id'],];
        }
        return $options;
    }

    public function getAttributeDetails()
    {
        return [];
    }

    public function getAttributesOptions($cat)
    {
        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $collection = $mongo->getCollectionForTable('product_container');

        $aggregate = [];
        $aggregate[] = [
            '$match' => [
                'user_id' => $this->di->getUser()->id
            ]
        ];
        $aggregate[] = [
            '$unwind' => '$variant_attributes'
        ];
        $aggregate[] = [
            '$group' => [
                '_id' => '$variant_attributes'
            ]
        ];
        $attr = json_decode(json_encode($collection->aggregate($aggregate)->toArray()), true);

        $newAttrOpt = [];

        $color = false;
        $size = false;

        foreach( $attr as $id ) {
            if ( $id['_id'] === 'color' ) {
                $color= true;
            }
            if ( $id['_id'] === 'size' ) {
                $size = true;
            }
            $newAttrOpt[] = [ 
                "label" => $id['_id'],
                "value" => $id['_id']
            ];
        }

        if ( !$color ) {
            $newAttrOpt[] = [ 
                "label" => 'color',
                "value" => 'color'
            ];
        }
        if ( !$size ) {
            $newAttrOpt[] = [ 
                "label" => 'size',
                "value" => 'size'
            ];
        }


        return [
            'attr' => [
                [ "label" => 'Title', "value" =>'title' ],
            [ "label" => 'SKU', "value" =>'sku' ],
            [ "label" => 'Brand', "value" =>'brand' ],
            ["label" => 'Product id',"value" =>'source_product_id',],
            [ "label" => 'Quantity', "value" =>'quantity' ],
            [ "label" => 'Weight', "value" =>'weight' ],
            [ "label" => 'Weight unit', "value" =>'weight_unit' ],
            [ "label" => 'Color', "value" =>'color' ],
            [ "label" => 'Barcode', "value" =>'barcode' ],
            [ "label" => 'Size', "value" =>'size' ],
            [ "label" => 'Height', "value" =>'height' ],
            [ "label" => 'Weight', "value" =>'weight' ],
            [ "label" => 'Length', "value" =>'length' ],
            [ "label" => 'Width', "value" =>'width' ],
            ],
            'variant_attr' => $newAttrOpt
        ];
    }

    public function initWebhook($data)
    {
        $actionToPerform = $data['data']['scope'];
        
        echo $actionToPerform;
        echo PHP_EOL;

        $getConnectedChannels = $this->getSellerConnectedChannels($data);
        $userId = $data['user_id'];
        switch ($actionToPerform) {
            case 'store/product/updated':
                //fetch product from BigCommerce

                $prepareProduct = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Route\Requestcontrol')->fetchBigcommerceProductById($data);
                if (isset($prepareProduct['data']) && !empty($prepareProduct['data'])) {
                    //send request to BigCommerce Base
                    $response = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Hook')->CreateOrUpdateProduct($prepareProduct, $userId, $data);
                    
                    if (!empty($response)) {
                        //send request to Connect Channels
                        if (!empty($getConnectedChannels)) {
                            foreach ($getConnectedChannels as $moduleHome => $channel) {
                                $class = '\App\\'.$moduleHome.'\Components\Product\Hook';
                                if (class_exists($class)) {
                                    if (method_exists($class, 'createQueueToUpdateProduct')) {
                                        $scheduleProcess = $this->di->getObjectManager()->get($class)->createQueueToUpdateProduct($data);
                                    } else {
                                        $this->di->getLog()->logContent('createQueueToUpdateProduct Class found, method not found. '.json_encode($data), 'info', 'MethodNotFound.log');
                                    }
                                } else {
                                    $this->di->getLog()->logContent($class.' Class not found. '.json_encode($data), 'info', 'ClassNotFound.log');
                                }
                            }
                        }
                    }
                }
                break;
            case 'store/product/created':
                //fetch product from BigCommerce
                $prepareProduct = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Route\Requestcontrol')->fetchBigcommerceProductById($data);
                if (isset($prepareProduct['data']) && !empty($prepareProduct['data'])) {
                    //send request to BigCommerce Base
                    $response = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Hook')->CreateOrUpdateProduct($prepareProduct, $userId, $data);
                    return true;                    
                    if (!empty($response)) {
                        //send request to Connect Channels
                        if (!empty($getConnectedChannels)) {
                            foreach ($getConnectedChannels as $moduleHome => $channel) {
                                $class = '\App\\'.$moduleHome.'\Components\Product\Hook';
                                if (class_exists($class)) {
                                    if (method_exists($class, 'createQueueToCreateProduct')) {
                                        $scheduleProcess = $this->di->getObjectManager()->get($class)->createQueueToCreateProduct($data);
                                    } else {
                                        $this->di->getLog()->logContent('createQueueToCreateProduct Class found, method not found. '.json_encode($data), 'info', 'MethodNotFound.log');
                                    }
                                } else {
                                    $this->di->getLog()->logContent($class.' Class not found. '.json_encode($data), 'info', 'ClassNotFound.log');
                                }
                            }
                        }
                    }
                }
                break;

            case 'store/product/deleted':
                //send request to BigCommerce Base
                $response = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Hook')->deleteProduct($userId, $data);
                // return true;
                if (!empty($response)) {
                    $product = $response['product'];
                    //send request to Connect Channels
                    if (!empty($getConnectedChannels)) {
                        foreach ($getConnectedChannels as $moduleHome => $channel) {
                            $class = '\App\\'.$moduleHome.'\Components\Product\Hook';
                            if (class_exists($class)) {
                                if (method_exists($class, 'createQueueToDeleteProduct')) {
                                    $scheduleProcess = $this->di->getObjectManager()->get($class)->createQueueToDeleteProduct($data, $product);
                                } else {
                                    $this->di->getLog()->logContent('createQueueToDeleteProduct Class found, method not found. '.json_encode($data), 'info', 'MethodNotFound.log');
                                }
                            } else {
                                $this->di->getLog()->logContent($class.' Class not found. '.json_encode($data), 'info', 'ClassNotFound.log');
                            }
                        }
                    }
                }
                break;

            case 'app_delete':
                $this->di->getObjectManager()->get("\App\Shopifyhome\Components\Core\Hook")->uninstall();
                break;

            case 'store/product/inventory/updated':
                //fetch product from BigCommerce
                $prepareProduct = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Route\Requestcontrol')->fetchBigcommerceProductById($data);
                if (isset($prepareProduct['data']) && !empty($prepareProduct['data'])) {
                    //send request to BigCommerce Base
                    $response = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Hook')->CreateOrUpdateProduct($prepareProduct, $userId, $data);
                    // return true;
                    if (!empty($response)) {
                        //send request to Connect Channels
                        if (!empty($getConnectedChannels)) {
                            foreach ($getConnectedChannels as $moduleHome => $channel) {
                                $class = '\App\\'.$moduleHome.'\Components\Product\Hook';
                                if (class_exists($class)) {
                                    if (method_exists($class, 'createQueueToUpdateProductInventory')) {
                                        $scheduleProcess = $this->di->getObjectManager()->get($class)->createQueueToUpdateProductInventory($data);
                                    } else {
                                        $this->di->getLog()->logContent('createQueueToUpdateProductInventory Class found, method not found. '.json_encode($data), 'info', 'MethodNotFound.log');
                                    }
                                } else {
                                    $this->di->getLog()->logContent($class.' Class not found. '.json_encode($data), 'info', 'ClassNotFound.log');
                                }
                            }
                        }
                    }
                }
                break;
            case 'store/product/inventory/order/updated':
                //fetch product from BigCommerce
                $prepareProduct = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Route\Requestcontrol')->fetchBigcommerceProductById($data);
                if (isset($prepareProduct['data']) && !empty($prepareProduct['data'])) {
                    //send request to BigCommerce Base
                    $response = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Hook')->CreateOrUpdateProduct($prepareProduct, $userId, $data);
                    
                    if (!empty($response)) {
                        //send request to Connect Channels
                        if (!empty($getConnectedChannels)) {
                            foreach ($getConnectedChannels as $moduleHome => $channel) {
                                $class = '\App\\'.$moduleHome.'\Components\Product\Hook';
                                if (class_exists($class)) {
                                    if (method_exists($class, 'createQueueToUpdateProductInventory')) {
                                        $scheduleProcess = $this->di->getObjectManager()->get($class)->createQueueToUpdateProductInventory($data);
                                    } else {
                                        $this->di->getLog()->logContent('createQueueToUpdateProductInventory Class found, method not found. '.json_encode($data), 'info', 'MethodNotFound.log');
                                    }
                                } else {
                                    $this->di->getLog()->logContent($class.' Class not found. '.json_encode($data), 'info', 'ClassNotFound.log');
                                }
                            }
                        }
                    }
                }
                break;

            case 'store/sku/updated':
                //fetch product from BigCommerce
                $prepareProduct = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Route\Requestcontrol')->fetchBigcommerceProductBySku($data);
                if (isset($prepareProduct['data']) && !empty($prepareProduct['data'])) {
                    //send request to BigCommerce Base
                    $response = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Hook')->CreateOrUpdateSku($prepareProduct, $userId, $data);
                    if (!empty($response)) {
                        //send request to Connect Channels
                        if (!empty($getConnectedChannels)) {
                            foreach ($getConnectedChannels as $moduleHome => $channel) {
                                $class = '\App\\'.$moduleHome.'\Components\Product\Hook';
                                if (class_exists($class)) {
                                    if (method_exists($class, 'createQueueToUpdateSku')) {
                                        $scheduleProcess = $this->di->getObjectManager()->get($class)->createQueueToUpdateSku($data);
                                    } else {
                                        $this->di->getLog()->logContent('createQueueToUpdateSku Class found, method not found. '.json_encode($data), 'info', 'MethodNotFound.log');
                                    }
                                } else {
                                    $this->di->getLog()->logContent($class.' Class not found. '.json_encode($data), 'info', 'ClassNotFound.log');
                                }
                            }
                        }
                    }
                }
                break;
            case 'store/sku/created':
                //fetch product from BigCommerce
                $prepareProduct = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Route\Requestcontrol')->fetchBigcommerceProductBySku($data);
                if (isset($prepareProduct['data']) && !empty($prepareProduct['data'])) {
                    //send request to BigCommerce Base
                    $response = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Hook')->CreateOrUpdateSku($prepareProduct, $userId, $data);
                    return true;
                    if (!empty($response)) {
                        //send request to Connect Channels
                        if (!empty($getConnectedChannels)) {
                            foreach ($getConnectedChannels as $moduleHome => $channel) {
                                $class = '\App\\'.$moduleHome.'\Components\Product\Hook';
                                if (class_exists($class)) {
                                    if (method_exists($class, 'createQueueToCreateSku')) {
                                        $scheduleProcess = $this->di->getObjectManager()->get($class)->createQueueToCreateSku($data);
                                    } else {
                                        $this->di->getLog()->logContent('createQueueToCreateSku Class found, method not found. '.json_encode($data), 'info', 'MethodNotFound.log');
                                    }
                                } else {
                                    $this->di->getLog()->logContent($class.' Class not found. '.json_encode($data), 'info', 'ClassNotFound.log');
                                }
                            }
                        }
                    }
                }
                break;

            case 'store/sku/deleted':
                //send request to BigCommerce Base
                $response = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Hook')->deleteSku($userId, $data);
                // return true;
                if (!empty($response)) {
                    $product = $response['product'];
                    //send request to Connect Channels
                    if (!empty($getConnectedChannels)) {
                        foreach ($getConnectedChannels as $moduleHome => $channel) {
                            $class = '\App\\'.$moduleHome.'\Components\Product\Hook';
                            if (class_exists($class)) {
                                if (method_exists($class, 'createQueueToDeleteProduct')) {
                                    $scheduleProcess = $this->di->getObjectManager()->get($class)->createQueueToDeleteSku($data, $product);
                                } else {
                                    $this->di->getLog()->logContent('createQueueToDeleteProduct Class found, method not found. '.json_encode($data), 'info', 'MethodNotFound.log');
                                }
                            } else {
                                $this->di->getLog()->logContent($class.' Class not found. '.json_encode($data), 'info', 'ClassNotFound.log');
                            }
                        }
                    }
                }
                break;
            case 'store/sku/inventory/updated':
                //fetch product from BigCommerce
                $prepareProduct = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Route\Requestcontrol')->fetchBigcommerceProductBySku($data);
                if (isset($prepareProduct['data']) && !empty($prepareProduct['data'])) {
                    //send request to BigCommerce Base
                    $response = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Hook')->CreateOrUpdateSku($prepareProduct, $userId, $data);
                    // return true;
                    if (!empty($response)) {
                        //send request to Connect Channels
                        if (!empty($getConnectedChannels)) {
                            foreach ($getConnectedChannels as $moduleHome => $channel) {
                                $class = '\App\\'.$moduleHome.'\Components\Product\Hook';
                                if (class_exists($class)) {
                                    if (method_exists($class, 'createQueueToUpdateSkuInventory')) {
                                        $scheduleProcess = $this->di->getObjectManager()->get($class)->createQueueToUpdateSkuInventory($data);
                                    } else {
                                        $this->di->getLog()->logContent('createQueueToUpdateSkuInventory Class found, method not found. '.json_encode($data), 'info', 'MethodNotFound.log');
                                    }
                                } else {
                                    $this->di->getLog()->logContent($class.' Class not found. '.json_encode($data), 'info', 'ClassNotFound.log');
                                }
                            }
                        }
                    }
                }
                break;
            case 'store/sku/inventory/order/updated':
                //fetch product from BigCommerce
                $prepareProduct = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Route\Requestcontrol')->fetchBigcommerceProductBySku($data);
                if (isset($prepareProduct['data']) && !empty($prepareProduct['data'])) {
                    //send request to BigCommerce Base
                    $response = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Product\Hook')->CreateOrUpdateSku($prepareProduct, $userId, $data);
                    return true;
                    if (!empty($response)) {
                        //send request to Connect Channels
                        if (!empty($getConnectedChannels)) {
                            foreach ($getConnectedChannels as $moduleHome => $channel) {
                                $class = '\App\\'.$moduleHome.'\Components\Product\Hook';
                                if (class_exists($class)) {
                                    if (method_exists($class, 'createQueueToUpdateSkuInventory')) {
                                        $scheduleProcess = $this->di->getObjectManager()->get($class)->createQueueToUpdateSkuInventory($data);
                                    } else {
                                        $this->di->getLog()->logContent('createQueueToUpdateSkuInventory Class found, method not found. '.json_encode($data), 'info', 'MethodNotFound.log');
                                    }
                                } else {
                                    $this->di->getLog()->logContent($class.' Class not found. '.json_encode($data), 'info', 'ClassNotFound.log');
                                }
                            }
                        }
                    }
                }
                break;
            case 'store/information/updated':
                $this->di->getLog()->logContent(json_encode($data), 'info', 'storeWebhook.log');
                //fetch store Details from BigCommerce
                $storeDetails = $this->di->getObjectManager()->get('\App\Bigcommercehome\Models\SourceModel')->fetchAndUpdateBigCommerceStoreDetails($userId, $data);
                return true;
                break;
        }

        return true;
    }

    public function fetchAndUpdateBigCommerceStoreDetails($userId, $data){
        $appCode = $data['app_code'];
        $data = $data['data'];
        $target = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Shop')->getUserMarkeplace();
        $remoteShopId = $this->di->getObjectManager()->get("\App\Core\Models\User\Details")->getDataByUserID($userId, $target)->remote_shop_id;
        $shopResponse = $this->di->getObjectManager()->get('\App\Connector\Components\ApiClient')->init($target, true, $appCode)->call('shop', [], ['shop_id' => $remoteShopId]);
        print_r(json_encode($shopResponse));
        if(isset($shopResponse['success'])){
            $storeDetails = $shopResponse['data'];

            $currency = $storeDetails['currency'];
            $currencySymbol = $storeDetails['currency_symbol'];
            $weightUnits = $storeDetails['weight_units'];
            $dimensionUnits = $storeDetails['dimension_units'];

            $mongo = $this->di->getObjectManager()->get('\App\Core\Models\BaseMongo');
            $userCollection = $mongo->getCollectionForTable("user_details");
            // $existingStore = $collection->findOne(['user_id' => $userId]);

            $userData = $this->di->getObjectManager()->get("\App\Core\Models\User\Details")->getDataByUserID($userId, $target);
            if($userData['currency'] != $currency){
                echo "currency_update";
                echo PHP_EOL;
                $userCollection->updateOne(['user_id'=> $userId], ['$set' => ['shops.0.currency' => (string)$currency ]]);
                $userCollection->updateOne(['user_id'=> $userId], ['$set' => ['shops.0.currency_symbol' => (string)$currencySymbol ]]);

                $productCollection = $mongo->getCollectionForTable("product_container");
                $productCollection->updateMany(['user_id'=> $userId], ['$set' => ['currency' => (string)$currency]]);
                //update currency to user_details & product container
            }

            if($userData['weight_units'] != $weightUnits){
                echo "weight_units_update";
                echo PHP_EOL;
                $userCollection->updateOne(['user_id'=> $userId], ['$set' => ['shops.0.weight_units' => (string)$weightUnits ]]);
                 
                $productWeightUnits = $this->di->getObjectManager()->get("\App\Bigcommercehome\Components\Product\Helper")->formatWeightUnits($weightUnits);
                //update weight_units to user_details & in products

                $productCollection = $mongo->getCollectionForTable("product_container");
                $productCollection->updateMany(['user_id'=> $userId], ['$set' => ['weight_units' => (string)$productWeightUnits, 'original_weight_units' => (string)$weightUnits]]);
            }

            if($userData['dimension_units'] != $dimensionUnits){
                echo "dimension_units_update";
                echo PHP_EOL;
                $userCollection->updateOne(['user_id'=> $userId], ['$set' => ['shops.0.dimension_units' => (string)$dimensionUnits ]]);
                //update dimension_units to user_details & in products

                $productDimensionUnits = $this->di->getObjectManager()->get("\App\Bigcommercehome\Components\Product\Helper")->formatDimensionUnits($dimensionUnits);

                $productCollection = $mongo->getCollectionForTable("product_container");
                $productCollection->updateMany(['user_id'=> $userId], ['$set' => ['dimension_units' => (string)$productDimensionUnits, 'original_dimension_units' => (string)$dimensionUnits]]);
            }

        }
        return true;
    }

    public function registerWebhooks($source)
    {
        $target = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Shop')->getUserMarkeplace();
        $userId = $this->di->getUser()->id;
        $user_details = $this->di->getObjectManager()->get('\App\Core\Models\User\Details');
        $user_details = $user_details->getDataByUserID($userId, $target);

        $awsConfig = include_once(BP . DS . 'app' . DS . 'etc' . DS . 'aws.php');
        $sqsConfig = [
            'region' => $awsConfig['region'],
            'key' => $awsConfig['credentials']['key'],
            'secret' => $awsConfig['credentials']['secret']
        ];
        
        $handlerData = [
            'type' => 'full_class',
            'class_name' => '\App\Bigcommercehome\Components\Webhook\Route\Requestcontrol',
            'method' => 'registerWebhooks',
            'queue_name' => 'bigcommerce_register_webhook',
            'user_id' => $userId,
            'data' => [
                'user_id' => $userId,
                'remote_shop_id' => $user_details['remote_shop_id'],
                'sqs' => $sqsConfig,
                'cursor' => 0,
                'source' => $source,
                'target' => $target
            ]
        ];
        $rmqHelper = $this->di->getObjectManager()->get('\App\Rmq\Components\Helper');

        $this->di->getLog()->logContent('webhook SQS data =' . json_encode($handlerData), 'info', 'bigcommerce'.DS.$this->di->getUser()->id.DS.'webhook_register.log');

        return [
            'success' => true,
            'message' => 'Registering Webhooks ...',
            'queue_sr_no' => $rmqHelper->createQueue($handlerData['queue_name'], $handlerData)
        ];
    }

    public function registerUninstallWebhook($appCode)
    {
        $target = $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Shop\Shop')->getUserMarkeplace();
        $userId = $this->di->getUser()->id;
        $user_details = $this->di->getObjectManager()->get('\App\Core\Models\User\Details');
        $user_details = $user_details->getDataByUserID($userId, $target);

        $awsConfig = include_once(BP . DS . 'app' . DS . 'etc' . DS . 'aws.php');

        $webhookDataForAppDelete = [
            'type' => 'sqs',
            'queue_config' => [
                'region' => $awsConfig['region'],
                'key' => $awsConfig['credentials']['key'],
                'secret' => $awsConfig['credentials']['secret']
            ],
            'topic' => 'app/uninstalled',
            'queue_unique_id' => 'sqs-bigcommerce-app-uninstalled-' . $userId,
            'format' => 'json',
            'queue_name' => 'bigcommerce_webhook_app_delete',
            'queue_data' => [
                'type' => 'full_class',
                'class_name' => '\App\Connector\Models\SourceModel',
                'method' => 'triggerWebhooks',
                'user_id' => $userId,
                'code' => 'Bigcommercehome',
                'action' => 'app_delete',
                'queue_name' => 'webhook_app_delete'
            ]
        ];

        $responseWbhook = $this->di->getObjectManager()->get("\App\Connector\Components\ApiClient")
            ->init($target, 'true', $appCode)
            ->call('/webhook/register', [], ['shop_id' => $user_details['remote_shop_id'], 'data' => $webhookDataForAppDelete], 'POST');

        $this->di->getLog()->logContent('uninstall webhook triggerred for user id : ' . $userId . ' | response :' . json_encode($responseWbhook), 'info', 'uninstallWebhook.log');
    }

    // function of unregistering product create webhook for all user_ids
    public function productCreateUnregisterWebhook($marketplaceDetails = [], $userId = null)
    {
        $handlerData = [
            'type' => 'full_class',
            'class_name' => '\App\Bigcommercehome\Components\Shop\Route\Requestcontrol',
            'method' => 'UnregisterProductCreateWebhooks',
            'queue_name' => 'webhook_product_create',
            'user_id' => $userId,
            'data' => [
                'topic' =>'products/create',
                'user_id' => $userId,
                'chunkSize' => 20,
                'cursor' => 0
            ]
        ];
        $rmqHelper = $this->di->getObjectManager()->get('\App\Rmq\Components\Helper');
        return [
            'success' => true,
            'message' => 'Unregister Webhooks Process Initiated',
            'queue_sr_no' => $rmqHelper->createQueue($handlerData['queue_name'], $handlerData)
        ];
    }

    public function getInstallationForm()
    {
        $apiConnector = $this->di->getConfig()->get('apiconnector');
        return [
            'post_type' => 'view_template',
            'viewDir' => '/bigcommercehome/view/',
            'template' => 'install',
            'params' => [
                'url' => $apiConnector->base_url . 'apiconnect/request/auth',
                'query_params' => [
                    'sAppId' => $apiConnector->bigcommerce->sub_app_id ,
                    'state' => $this->di->getUser()->id,
                ]
            ],
        ];
    }

    /*
    @ Order Management Starts - 170321
    */
    public function initiateOrderImport($marketplaceDetails = [], $userId = null, $force=false)
    {
        $params = $this->di->getRequest()->get();
        $userId = $userId ?? $this->di->getUser()->id;
        if (!$userId) {
            return [ 'success' => false, 'message' => 'Invalid User. Please check your login credentials' ];
        }

        $shop = $this->di->getObjectManager()->get("\App\Core\Models\User\Details")->getDataByUserID($userId, 'bigcommerce');
        if (isset($shop['message'])) {
            return $shop;
        }
        $remoteShopId = $shop->remote_shop_id;

        $queuedTaskModel = $this->di->getObjectManager()->get('\App\Connector\Models\QueuedTasks');
        $queuedTaskModel->getCollection();
        if (!$queuedTaskModel->count([
            'user_id' => $userId,
            'type' => 'order_import',
            'shop_id' => (string) $shop['_id'],
        ])) {
            if (!$force) {
                return ['success' => false, 'message' => 'Order Import process is already under progress. Please check notification for updates.'];
            }
        } else {
            $queuedTask = new \App\Connector\Models\QueuedTasks;
            $queuedTask->set([
                'user_id' => (int)$userId,
                'message' =>'initializing Order Import...',
                'type' => 'order_import',
                'progress' => 0.00,
                'shop_id' => (string) $shop['_id'],
            ]);
            $status = $queuedTask->save();

            $handlerData = $this->di->getObjectManager()->get("\App\Bigcommercehome\Components\Order\Helper")->shapeSqsData($userId, $remoteShopId, $queuedTask);

            $rmqHelper = $this->di->getObjectManager()->get('\App\Rmq\Components\Helper');

            if (empty($append)) {
                $prefix = '';
            } else {
                $prefix = 'vistar_';
            }
            $this->di->getLog()->logContent('*********************************************************************', 'info', 'bigcommerce'.DS.$userId.DS.'order'.DS.date("Y-m-d").DS.$prefix.'order_import.log');
            $this->di->getLog()->logContent('PROCESS 0 CLASS : SourceModel.php | FUNCTION : initiateImport | Start', 'info', 'bigcommerce'.DS.$userId.DS.'order'.DS.date("Y-m-d").DS.$prefix.'order_import.log');
            $this->di->getLog()->logContent('*********************************************************************', 'info', 'bigcommerce'.DS.$userId.DS.'order'.DS.date("Y-m-d").DS.$prefix.'order_import.log');
            return [
                'success' => true,
                'message' => 'Bigcommerce Order Import Initiated',
                'queue_sr_no' => $rmqHelper->createQueue($handlerData['queue_name'], $handlerData)
            ];
        }
    }
  
  
     
    public function updateOrder($updateData, $userId = false)
    {
        return $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Order\Data')->handleOrderData($updateData, $userId);
    }

    /*
    @ Customer Management Starts - 170321
    */
    public function initiateCustomerImport($marketplaceDetails = [], $userId = null, $force=false)
    {
        $params = $this->di->getRequest()->get();
        $userId = $userId ?? $this->di->getUser()->id;
        if (!$userId) {
            return [ 'success' => false, 'message' => 'Invalid User. Please check your login credentials' ];
        }

        $shop = $this->di->getObjectManager()->get("\App\Core\Models\User\Details")->getDataByUserID($userId, 'bigcommerce');
        if (isset($shop['message'])) {
            return $shop;
        }
        $remoteShopId = $shop->remote_shop_id;
        $queuedTaskModel = $this->di->getObjectManager()->get('\App\Connector\Models\QueuedTasks');
        $queuedTaskModel->getCollection();
        if (!$queuedTaskModel->count([
            'user_id' => $userId,
            'type' => 'customer_import',
            'shop_id' => (string) $shop['_id'],
        ])) {
            if (!$force) {
                return ['success' => false, 'message' => 'Customer Import process is already under progress. Please check notification for updates.'];
            }
        } else {
            $queuedTask = new \App\Connector\Models\QueuedTasks;
            $queuedTask->set([
                'user_id' => (int)$userId,
                'message' =>'initializing Customer Import...',
                'type' => 'customer_import',
                'progress' => 0.00,
                'shop_id' => (string) $shop['_id'],
            ]);
            $status = $queuedTask->save();

            $handlerData = $this->di->getObjectManager()->get("\App\Bigcommercehome\Components\Customer\Helper")->shapeSqsData($userId, $remoteShopId, $queuedTask);

            $rmqHelper = $this->di->getObjectManager()->get('\App\Rmq\Components\Helper');

            if (empty($append)) {
                $prefix = '';
            } else {
                $prefix = 'vistar_';
            }
            $this->di->getLog()->logContent('*********************************************************************', 'info', 'bigcommerce'.DS.$userId.DS.'order'.DS.date("Y-m-d").DS.$prefix.'customer_import.log');
            $this->di->getLog()->logContent('PROCESS 0 CLASS : SourceModel.php | FUNCTION : initiateImport | Start', 'info', 'bigcommerce'.DS.$userId.DS.'order'.DS.date("Y-m-d").DS.$prefix.'customer_import.log');
            $this->di->getLog()->logContent('*********************************************************************', 'info', 'bigcommerce'.DS.$userId.DS.'order'.DS.date("Y-m-d").DS.$prefix.'customer_import.log');
            return [
                'success' => true,
                'message' => 'BigCommerce Customer Import Initiated',
                'queue_sr_no' => $rmqHelper->createQueue($handlerData['queue_name'], $handlerData)
            ];
        }
    }

    public function updateCustomer($updateData, $userId = false)
    {
        return $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Customer\Data')->handleCustomerData($updateData, $userId);
    }

    /*
    @ Abandoned Carts Management Starts - 170321
    */
    public function initiateCartImport($marketplaceDetails = [], $userId = null, $force=false)
    {
        $params = $this->di->getRequest()->get();
        $userId = $userId ?? $this->di->getUser()->id;
        if (!$userId) {
            return [ 'success' => false, 'message' => 'Invalid User. Please check your login credentials' ];
        }

        $shop = $this->di->getObjectManager()->get("\App\Core\Models\User\Details")->getDataByUserID($userId, 'bigcommerce');
        if (isset($shop['message'])) {
            return $shop;
        }
        $remoteShopId = $shop->remote_shop_id;
        $queuedTaskModel = $this->di->getObjectManager()->get('\App\Connector\Models\QueuedTasks');
        $queuedTaskModel->getCollection();
        if (!$queuedTaskModel->count([
            'user_id' => $userId,
            'type' => 'cart_import',
            'shop_id' => (string) $shop['_id'],
        ])) {
            if (!$force) {
                return ['success' => false, 'message' => 'Checkout Import process is already under progress. Please check notification for updates.'];
            }
        } else {
            $queuedTask = new \App\Connector\Models\QueuedTasks;
            $queuedTask->set([
                'user_id' => (int)$userId,
                'message' =>'initializing Checkout Import...',
                'type' => 'cart_import',
                'progress' => 0.00,
                'shop_id' => (string) $shop['_id'],
            ]);
            $status = $queuedTask->save();

            $handlerData = $this->di->getObjectManager()->get("\App\Bigcommercehome\Components\Cart\Helper")->shapeSqsData($userId, $remoteShopId, $queuedTask);
            
            $rmqHelper = $this->di->getObjectManager()->get('\App\Rmq\Components\Helper');

            if (empty($append)) {
                $prefix = '';
            } else {
                $prefix = 'vistar_';
            }
            $this->di->getLog()->logContent('*********************************************************************', 'info', 'bigcommerce'.DS.$userId.DS.'order'.DS.date("Y-m-d").DS.$prefix.'cart_import.log');
            $this->di->getLog()->logContent('PROCESS 0 CLASS : SourceModel.php | FUNCTION : initiateImport | Start', 'info', 'bigcommerce'.DS.$userId.DS.'order'.DS.date("Y-m-d").DS.$prefix.'cart_import.log');
            $this->di->getLog()->logContent('*********************************************************************', 'info', 'bigcommerce'.DS.$userId.DS.'order'.DS.date("Y-m-d").DS.$prefix.'cart_import.log');
            return [
                'success' => true,
                'message' => 'BigCommerce Cart Import Initiated',
                'queue_sr_no' => $rmqHelper->createQueue($handlerData['queue_name'], $handlerData)
            ];
        }
    }

    public function updateCart($updateData, $userId = false)
    {
        return $this->di->getObjectManager()->get('\App\Bigcommercehome\Components\Cart\Data')->handleCartData($updateData, $userId);
    }
    //Cart Management Ends

    public function getSellerConnectedChannels($data)
    {
        $returnDetails = [];
        $moduleCode = $data['code'];

        $shops = $this->di->getUser()->shops;
        
        foreach ($shops as $key => $shop) {
            if (is_array($shop)) {
                $shop['marketplacehome'] = $shop['marketplace'];
                if ($shop['marketplace']=='mercado_cbt') {
                    $shop['marketplacehome'] = 'mercado';
                }
                $moduleHome = ucfirst($shop['marketplacehome']).'home';
                if ($moduleHome != $moduleCode) {
                    $returnDetails[ucfirst($moduleHome)] = $shop;
                }
            }
        }
        return $returnDetails;
    }

    public function getChannelWebhooks($webhookIdentifier)
    {
        $returnArray = [];
        $webhookActions = $this->di->getConfig()->webhook->$webhookIdentifier;
        foreach ($webhookActions as $key => $webhook) {
            $returnArray[$webhook['topic']] = $webhook['action'];
        }

        return $returnArray;
    }
}
